from pydantic import model_validator
from typing import Any, Optional
from .base import Base


class Compact(Base):

    component: str = "compactField"
    """
    组件名称
    """

    block: bool = False
    """
    将宽度调整为父元素宽度的选项,默认值false"
    """

    direction: str = None
    """
    间距方向
    """

    size: str = None
    """
    间距大小
    """

    body: Optional[Any] = None
    """
    组件内容
    """

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        self.only_on_forms()
        return self

    def set_block(self, block: bool):
        """
        设置是否将宽度调整为父元素宽度。
        :param block: 是否调整的布尔值
        :return: 当前组件实例
        """
        self.block = block
        return self

    def set_direction(self, direction: str):
        """
        设置间距方向。
        :param direction: 间距方向字符串
        :return: 当前组件实例
        """
        self.direction = direction
        return self

    def set_size(self, size: str):
        """
        设置间距大小。
        :param size: 间距大小字符串
        :return: 当前组件实例
        """
        self.size = size
        return self

    def set_body(self, body: Any):
        """
        设置容器控件里面的内容。
        :param body: 组件内容
        :return: 当前组件实例
        """
        self.body = body
        return self



# coding: utf-8
import logging
import os
import shutil
import sys
from contextlib import asynccontextmanager
from typing import Any

import i18n
import uvicorn
from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles
from tortoise import Tortoise

from . import cache, config, db
from .install import setup_all
from .middleware import Middleware
from .routes import auth, dashboard, resource, upload


class Quark(FastAPI):

    # 配置
    config: dict[str, Any] = {
        "APP_NAME": "QuarkPy",
        "APP_VERSION": "0.2.0",
        "APP_SECRET_KEY": "your-secret-key",
        "CACHE_PREFIX": "quark-cache",
        "MODULE_PATH": "",
        "LOCALE": "zh-hans",
        "DB_CONFIG": None,
        "DB_URL": None,
        "DB_MODULES": {
            "models": [],
        },
    }

    def __init__(self, *args, **kwargs):
        """初始化"""
        super().__init__(*args, lifespan=self.lifespan, **kwargs)

        # 注册中间件
        self.register_middleware()

        # 日志
        self.logger = logging.getLogger(__name__)

        # 获取当前文件的绝对路径
        self.current_dir_path = os.path.dirname(os.path.abspath(__file__))

    def sync_config(self) -> None:
        """同步配置到全局变量"""
        config.init(self.config)

    def init_cache(self) -> None:
        """初始化缓存"""
        cache.init(self.config["CACHE_PREFIX"])

    # 初始化数据库
    async def init_db(self) -> None:
        """初始化数据库"""
        self.config["DB_MODULES"]["models"] = self.config["DB_MODULES"]["models"] + [
            "quark.models"
        ]
        await db.init(
            config=self.config["DB_CONFIG"],
            db_url=self.config["DB_URL"],
            modules=self.config["DB_MODULES"],
        )

    def init_locale(self) -> None:
        """初始化 locale"""
        locales_path = os.path.abspath(os.path.join(self.current_dir_path, "locales"))

        # 设置 locale 路径
        i18n.load_path.append(locales_path)

        # 设置默认语言
        i18n.set("locale", self.config["LOCALE"])

    def register_routers(self) -> None:
        """注册路由"""
        self.include_router(auth.router)
        self.include_router(dashboard.router)
        self.include_router(resource.router)
        self.include_router(upload.router)

    def register_middleware(self) -> None:
        """注册中间件"""
        self.add_middleware(Middleware)

    def load_static(self):
        """加载静态资源"""

        # 源目录
        src_static_path = os.path.join(self.current_dir_path, "web")

        # 目标目录
        dst_static_path = os.path.join(
            os.path.dirname(os.path.abspath(sys.argv[0])), "web"
        )

        # 复制文件夹
        if not os.path.exists(dst_static_path):
            shutil.copytree(src_static_path, dst_static_path)

        web_static_path = os.path.join(dst_static_path, "static")
        self.mount(
            "/static", StaticFiles(directory=web_static_path, html=True), name="static"
        )

        web_root_path = os.path.join(dst_static_path, "app")
        self.mount("/", StaticFiles(directory=web_root_path, html=True), name="web")

    @asynccontextmanager
    async def lifespan(self, app: FastAPI):
        """生命周期"""

        # 应用启动时执行
        await self.startup()

        yield

        # 应用关闭时执行
        await self.shutdown()

    async def startup(self) -> Any:
        """启动服务"""

        # 同步配置到全局变量
        self.sync_config()

        # 初始化缓存
        self.init_cache()

        # 初始化 locale
        self.init_locale()

        # 注册路由
        self.register_routers()

        # 设置静态资源
        self.load_static()

        # 初始化数据库
        await self.init_db()

        # 安装应用
        await setup_all()

    async def shutdown(self) -> Any:
        """关闭服务"""
        await Tortoise.close_connections()

    def run(
        self,
        app: Any | None = None,
        host: str | None = None,
        port: int | None = None,
        reload: bool | None = None,
    ) -> None:
        """uvicorn启动应用"""

        # 获取应用
        if app is None:
            app = self

        # 启动服务
        uvicorn.run(
            app=app, host=host or "127.0.0.1", port=port or 8000, reload=reload or False
        )



from pydantic import Field, model_validator
from typing import Any, Dict, Optional
from ...component import Component

class Text(Component):
    component: str = "text"
    label: str = Field("", description="内容的描述")
    tooltip: str = Field("", description="内容的补充描述，hover 后显示")
    ellipsis: bool = Field(False, description="是否自动缩略")
    copyable: bool = Field(False, description="是否支持复制")
    span: int = Field(1, description="列数")
    value_type: str = Field("text", description="值类型")
    value_enum: str = Field("", description="值枚举")
    data_index: str = Field("", description="索引")
    value: Optional[Any] = Field(None, description="设置保存值")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self
    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_label(self, label: str):
        # 设置内容的描述
        self.label = label
        return self

    def set_tooltip(self, tooltip: str):
        # 设置内容的补充描述，hover 后显示
        self.tooltip = tooltip
        return self

    def set_ellipsis(self, ellipsis: bool):
        # 设置是否自动缩略
        self.ellipsis = ellipsis
        return self

    def set_copyable(self, copyable: bool):
        # 设置是否支持复制
        self.copyable = copyable
        return self

    def set_span(self, span: int):
        # 设置列数
        self.span = span
        return self

    def set_value_type(self, value_type: str):
        # 设置值类型
        self.value_type = value_type
        return self

    def set_value_enum(self, value_enum: str):
        # 设置值枚举
        self.value_enum = value_enum
        return self

    def set_data_index(self, data_index: str):
        # 设置索引
        self.data_index = data_index
        return self

    def set_value(self, value: Any):
        # 设置保存值
        self.value = value
        return self


import json
from typing import Any, Dict

from tortoise.models import Model

from quark import Message, Request

from ..performs_validation import PerformsValidation


class StoreRequest:

    # 请求对象
    request: Request

    # 资源对象
    resource: Any

    # 查询对象
    model: Model

    # 字段列表
    fields: list

    def __init__(self, request: Request, resource: Any, model: Model, fields: list):
        self.request = request
        self.resource = resource
        self.model = model
        self.fields = fields

    async def handle(self, data: Dict[str, Any]) -> Any:
        # 验证数据合法性
        try:
            await PerformsValidation(
                request=self.request, fields=self.fields
            ).validator_for_creation(data)
        except Exception as e:
            return Message.error(str(e))

        # 保存前回调
        try:
            data = await self.resource.before_saving(self.request, data)
        except Exception as e:
            return Message.error(str(e))

        # 重组数据
        new_data = {}
        model_fields = set(self.model._meta.fields_map.keys())

        for k, v in data.items():
            # 只处理模型中存在的字段
            if k in model_fields:
                # 处理特殊数据类型
                if isinstance(v, (list, dict)):
                    new_data[k] = json.dumps(v, ensure_ascii=False)
                else:
                    new_data[k] = v

        # 保存数据
        try:
            instance = await self.model.create(**new_data)

        except Exception as e:
            return Message.error(str(e))

        id = getattr(instance, "id", None)
        if not id:
            return Message.error("参数错误")

        # 保存后回调
        try:
            await self.resource.after_saved(self.request, id, data, self.model)
        except Exception as e:
            return Message.error(str(e))

        return await self.resource.after_saved_redirect_to(self.request, id, data)



import hashlib
import uuid
from typing import Optional

from pydantic import BaseModel, ConfigDict


def to_camel(string: str) -> str:
    parts = string.split("_")
    return parts[0] + "".join(word.capitalize() for word in parts[1:])


class Component(BaseModel):

    # 组件配置
    model_config = ConfigDict(
        alias_generator=to_camel,
        populate_by_name=True,
        extra="allow",
    )

    def model_dump(self, exclude_none=True, **kwargs):
        """重写 model_dump 方法来默认排除 None 值"""
        return super().model_dump(exclude_none=exclude_none, **kwargs)

    # 组件名称
    component: Optional[str] = None

    # 组件key
    componentkey: Optional[str] = None

    # 组件样式
    style: Optional[dict] = None

    def set_component(self, component: str) -> "Component":
        self.component = component
        self.set_key(component, crypt=True)
        return self

    def set_key(self, key: Optional[str] = "", crypt: bool = True) -> "Component":
        if not key:
            key = str(uuid.uuid4())
        if crypt:
            md5_hash = hashlib.md5()
            md5_hash.update(key.encode("utf-8"))
            key = md5_hash.hexdigest()
        self.componentkey = key
        return self



from tortoise import fields
from tortoise.models import Model
from tortoise.transactions import in_transaction


class Menu(Model):

    id = fields.IntField(pk=True)
    name = fields.CharField(max_length=100)
    guard_name = fields.CharField(max_length=100)
    permission = fields.CharField(max_length=100)
    icon = fields.CharField(max_length=100, null=True)
    type = fields.IntField()
    page_type = fields.IntField()
    pid = fields.IntField(default=0)
    sort = fields.IntField(default=0)
    path = fields.CharField(max_length=255, null=True)
    query = fields.CharField(max_length=255, null=True)
    component = fields.CharField(max_length=255, null=True)
    visible = fields.IntField(default=1)
    status = fields.IntField(default=1)
    created_at = fields.DatetimeField(auto_now_add=True)
    updated_at = fields.DatetimeField(auto_now=True)

    class Meta(Model.Meta):
        table = "menus"

    def __str__(self):
        return f"<Menu {self.name}>"

    @staticmethod
    async def seeder():
        menus = [
            {
                "id": 1,
                "name": "首页",
                "guard_name": "admin",
                "icon": "ant-design:home-outlined",
                "type": 2,
                "pid": 0,
                "sort": 0,
                "path": "home",
                "query": '{"api":"/api/admin/dashboard/index/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 3,
                "name": "用户管理",
                "guard_name": "admin",
                "icon": "ant-design:usergroup-add-outlined",
                "type": 1,
                "pid": 0,
                "sort": 100,
                "path": "user",
                "query": "",
                "component": "",
                "visible": 1,
                "page_type": 1,
                "status": 1,
                "permission": "",
            },
            {
                "id": 4,
                "name": "用户列表",
                "guard_name": "admin",
                "icon": "ant-design:user-add-outlined",
                "type": 2,
                "pid": 3,
                "sort": 0,
                "path": "user",
                "query": '{"api":"/api/admin/user/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 5,
                "name": "权限列表",
                "guard_name": "admin",
                "icon": "ant-design:profile-outlined",
                "type": 2,
                "pid": 3,
                "sort": 0,
                "path": "permission",
                "query": '{"api":"/api/admin/permission/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 6,
                "name": "角色列表",
                "guard_name": "admin",
                "icon": "ant-design:idcard-outlined",
                "type": 2,
                "pid": 3,
                "sort": 0,
                "path": "role",
                "query": '{"api":"/api/admin/role/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 7,
                "name": "系统配置",
                "guard_name": "admin",
                "icon": "ant-design:setting-outlined",
                "type": 1,
                "pid": 0,
                "sort": 100,
                "path": "system",
                "query": "",
                "component": "",
                "visible": 1,
                "page_type": 1,
                "status": 1,
                "permission": "",
            },
            {
                "id": 8,
                "name": "设置管理",
                "guard_name": "admin",
                "icon": "ant-design:appstore-add-outlined",
                "type": 1,
                "pid": 7,
                "sort": 0,
                "path": "config",
                "query": "",
                "component": "",
                "visible": 1,
                "page_type": 1,
                "status": 1,
                "permission": "",
            },
            {
                "id": 9,
                "name": "网站设置",
                "guard_name": "admin",
                "icon": "ant-design:cluster-outlined",
                "type": 2,
                "pid": 8,
                "sort": 0,
                "path": "webConfig",
                "query": '{"api":"/api/admin/webConfig/form"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 10,
                "name": "配置管理",
                "guard_name": "admin",
                "icon": "ant-design:tool-outlined",
                "type": 2,
                "pid": 8,
                "sort": 0,
                "path": "config",
                "query": '{"api":"/api/admin/config/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 11,
                "name": "菜单管理",
                "guard_name": "admin",
                "icon": "ant-design:menu-outlined",
                "type": 2,
                "pid": 7,
                "sort": 0,
                "path": "menu",
                "query": '{"api":"/api/admin/menu/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 12,
                "name": "操作日志",
                "guard_name": "admin",
                "icon": "ant-design:file-done-outlined",
                "type": 2,
                "pid": 7,
                "sort": 100,
                "path": "actionLog",
                "query": '{"api":"/api/admin/actionLog/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 13,
                "name": "附件空间",
                "guard_name": "admin",
                "icon": "ant-design:folder-outlined",
                "type": 1,
                "pid": 0,
                "sort": 100,
                "path": "attachment",
                "query": "",
                "component": "",
                "visible": 1,
                "page_type": 1,
                "status": 1,
                "permission": "",
            },
            {
                "id": 14,
                "name": "文件管理",
                "guard_name": "admin",
                "icon": "ant-design:file-outlined",
                "type": 2,
                "pid": 13,
                "sort": 0,
                "path": "file",
                "query": '{"api":"/api/admin/file/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 15,
                "name": "图片管理",
                "guard_name": "admin",
                "icon": "ant-design:picture-outlined",
                "type": 2,
                "pid": 13,
                "sort": 0,
                "path": "image",
                "query": '{"api":"/api/admin/image/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 16,
                "name": "我的账号",
                "guard_name": "admin",
                "icon": "ant-design:user-outlined",
                "type": 1,
                "pid": 0,
                "sort": 110,
                "path": "account",
                "query": "",
                "component": "",
                "visible": 1,
                "page_type": 1,
                "status": 1,
                "permission": "",
            },
            {
                "id": 17,
                "name": "个人设置",
                "guard_name": "admin",
                "icon": "ant-design:user-switch-outlined",
                "type": 2,
                "pid": 16,
                "sort": 0,
                "path": "setting",
                "query": '{"api":"/api/admin/account/form"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 18,
                "name": "部门列表",
                "guard_name": "admin",
                "icon": "ant-design:apartment-outlined",
                "type": 2,
                "pid": 3,
                "sort": 0,
                "path": "department",
                "query": '{"api":"/api/admin/department/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 19,
                "name": "职位列表",
                "guard_name": "admin",
                "icon": "ant-design:bars-outlined",
                "type": 2,
                "pid": 3,
                "sort": 0,
                "path": "position",
                "query": '{"api":"/api/admin/position/index"}',
                "component": "",
                "visible": 1,
                "page_type": 2,
                "status": 1,
                "permission": "",
            },
            {
                "id": 20,
                "name": "组件调试",
                "guard_name": "admin",
                "icon": "ant-design:appstore-outlined",
                "type": 1,
                "pid": 0,
                "sort": 100,
                "path": "develop",
                "query": "",
                "component": "",
                "visible": 0,
                "page_type": 1,
                "status": 1,
                "permission": "",
            },
            {
                "id": 21,
                "name": "组件开发",
                "guard_name": "admin",
                "icon": "ant-design:experiment-outlined",
                "type": 2,
                "pid": 20,
                "sort": 0,
                "path": "index",
                "query": "",
                "component": "develop/index",
                "visible": 1,
                "page_type": 1,
                "status": 1,
                "permission": "",
            },
        ]

        # 在事务中逐条检查并插入
        async with in_transaction():
            for data in menus:
                if not await Menu.filter(id=data["id"]).exists():
                    await Menu.create(**data)



from .cascader import Cascader, Option
from .date import Date
from .date_range import DateRange
from .datetime import Datetime
from .datetime_range import DatetimeRange
from .input import Input
from .radio import Radio
from .search import Search
from .select import Select
from .tree_select import TreeSelect



from pydantic import Field, model_validator
from typing import Any, Dict, Optional
from ..component import Component


class Message(Component):
    component: str = Field(default="message")
    class_name: str = Field(default=None)
    type: str = Field(default="success")
    content: Any = Field(default=None)
    duration: int = Field(default=0)
    icon: str = Field(default="")
    style: Optional[Dict[str, Any]] = Field(default=None)
    data: Any = Field(default=None)
    url: str = Field(default="")

    # 返回成功
    @classmethod
    def success(cls, content, data=None, url=""):
        return (
            cls().set_type("success").set_content(content).set_url(url).set_data(data)
        )

    # 返回失败
    @classmethod
    def error(cls, content, url=""):
        return cls().set_type("error").set_content(content).set_url(url)

    @model_validator(mode="after")
    def init(self):
        self.component = "message"
        self.type = "success"
        self.set_key("message", True)

        return self

    # Set ClassName
    def set_class_name(self, class_name: str):
        self.class_name = class_name
        return self

    # Set Type info | success | error | warning | loading
    def set_type(self, message_type: str):
        self.type = message_type
        return self

    # Set style.
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    # 内容
    def set_content(self, content: Any):
        self.content = content
        return self

    # 自动关闭的延时，单位秒。设为 0 时不自动关闭
    def set_duration(self, duration: int):
        self.duration = duration
        return self

    # Set Icon
    def set_icon(self, icon: str):
        self.icon = icon
        return self

    # 设置返回数据
    def set_data(self, data: Any):
        self.data = data
        return self

    # 设置消息弹出后跳转链接
    def set_url(self, url: str):
        self.url = url
        return self



from typing import Any, Dict, List, Optional

from pydantic import Field, model_validator

from ..component import Component


class Search(Component):
    component: Optional[str] = "search"
    filter_type: Optional[str] = None
    search_text: Optional[str] = None
    reset_text: Optional[str] = None
    submit_text: Optional[str] = None
    label_width: Optional[int] = None
    span: Optional[int] = None
    class_name: Optional[str] = None
    default_collapsed: bool = True
    show_hidden_num: bool = False
    export_text: Optional[str] = None
    export_api: Optional[str] = None
    items: List[Any] = Field(default_factory=list)
    style: Optional[dict] = Field(default_factory=dict)

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        self.reset_text = "重置"
        self.submit_text = "搜索"
        return self

    # 设置方法（链式调用）
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_filter_type(self, filter_type: str):
        self.filter_type = filter_type
        return self

    def set_collapsed(self, collapsed: bool):
        self.default_collapsed = collapsed
        return self

    def set_search_text(self, search_text: str):
        self.search_text = search_text
        return self

    def set_reset_text(self, reset_text: str):
        self.reset_text = reset_text
        return self

    def set_submit_text(self, submit_text: str):
        self.submit_text = submit_text
        return self

    def set_class_name(self, class_name: str):
        self.class_name = class_name
        return self

    def set_label_width(self, label_width: int):
        self.label_width = label_width
        return self

    def set_span(self, span: int):
        self.span = span
        return self

    def set_split(self, show_hidden_num: bool):
        self.show_hidden_num = show_hidden_num
        return self

    def set_export_text(self, export_text: str):
        self.export_text = export_text
        return self

    def set_export_api(self, export_api: str):
        self.export_api = export_api
        return self

    def set_items(self, item: Any):
        self.items.append(item)
        return self



from pydantic import Field, model_validator
from typing import Any, Dict, Optional, Union
from ..component import Component

class Descriptions(Component):
    component: str = "descriptions"
    title: str = Field("", description="标题")
    tooltip: str = Field("", description="内容的补充描述，hover 后显示")
    extra: Optional[Any] = Field(None, description="描述列表的操作区域，显示在右上方")
    bordered: bool = Field(False, description="是否展示边框")
    column: Union[int, Dict[str, int]] = Field(1, description="一行的 ProDescriptionsItems 数量")
    size: str = Field("default", description="设置尺寸")
    layout: str = Field("horizontal", description="布局，horizontal|vertical")
    colon: bool = Field(True, description="配置 ProDescriptions.Item 的 colon 的默认值")
    columns: Optional[Any] = Field(None, description="列")
    data_source: Optional[Any] = Field(None, description="数据")
    init_api: Optional[Any] = Field(None, description="数据初始化接口")
    items: Optional[Any] = Field(None, description="数据项")
    actions: Optional[Any] = Field(None, description="行为")
    style: Optional[Dict[str, Any]] = Field(None, description="样式")

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    def set_style(self, style: Dict[str, Any]):
        # 设置样式
        self.style = style
        return self

    def set_title(self, title: str):
        # 设置标题
        self.title = title
        return self

    def set_tooltip(self, tooltip: str):
        # 设置内容的补充描述，hover 后显示
        self.tooltip = tooltip
        return self

    def set_extra(self, extra: Any):
        # 设置描述列表的操作区域
        self.extra = extra
        return self

    def set_bordered(self, bordered: bool):
        # 设置是否展示边框
        self.bordered = bordered
        return self

    def set_layout(self, layout: str):
        # 设置布局
        self.layout = layout
        return self

    def set_column(self, column: Union[int, Dict[str, int]]):
        # 设置一行的 ProDescriptionsItems 数量
        self.column = column
        return self

    def set_columns(self, columns: Any):
        # 设置列
        self.columns = columns
        return self

    def set_data_source(self, data_source: Any):
        # 设置数据
        self.data_source = data_source
        return self

    def set_size(self, size: str):
        # 设置尺寸
        self.size = size
        return self

    def set_colon(self, colon: bool):
        # 设置 ProDescriptions.Item 的 colon 的默认值
        self.colon = colon
        return self

    def set_init_api(self, api: Any):
        # 设置数据初始化接口
        self.init_api = api
        return self

    def set_items(self, items: Any):
        # 设置数据项
        self.items = items
        return self

    def set_actions(self, actions: Any):
        # 设置行为
        self.actions = actions
        return self


from ..models.position import Position
from quark.component.form.fields.checkbox import Option


class PositionService:
    def __init__(self):
        pass

    # 获取列表
    async def get_list(self):
        positions = await Position.all()
        options = [
            Option(label=position.name, value=position.id) for position in positions
        ]
        return options

    # 通过ID获取信息
    async def get_info_by_id(self, position_id: int):
        position = await Position.get(id=position_id, status=1)
        return position

    # 通过名称获取信息
    async def get_info_by_name(self, name: str):
        position = await Position.get(name=name, status=1)
        return position

    # 通过ID判断职位是否已存在
    async def is_exist(self, position_id: int):
        return await Position.exists(id=position_id)

    # 通过名称判断职位是否已存在
    async def is_exist_by_name(self, name: str):
        return await Position.exists(name=name)

    # 插入数据并返回ID
    async def insert_get_id(self, position: Position):
        await position.save()
        return position.id

    # 通过ID更新数据
    async def update_by_id(self, position_id: int, data: dict):
        position = await Position.get(id=position_id)
        for key, value in data.items():
            setattr(position, key, value)
        await position.save()

    # 通过ID删除记录
    async def delete_by_id(self, position_id: int):
        position = await Position.get(id=position_id)
        await position.delete()



from quark.component.statistic.statistic import Statistic
from quark.template.metric.value import Value
from quark.services.action_log import ActionLogService


class TotalLog(Value):
    title: str = "日志数量"
    col: int = 6

    async def calculate(self) -> Statistic:
        """计算数值"""
        count = await ActionLogService().count()
        return self.count(count).set_value_style({"color": "#999999"})



from typing import Any, Dict, List, Optional

from pydantic import model_validator

from .base import Base


class Icon(Base):

    component: str = "iconField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认的选中项
    """

    disabled: bool = False
    """
    整组失效，默认 False
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    placeholder: Optional[str] = None
    """
    占位符
    """

    size: str = ""
    """
    大小，large | middle | small，默认
    """

    allow_clear: bool = True
    """
    是否支持清除，默认 True
    """

    allow_search: bool = True
    """
    是否支持搜索，默认 True
    """

    options: List[str] = []
    """
    可选项数据源
    """

    @model_validator(mode="after")
    def init(self):
        self.options = [
		"ant-design:database-outlined", "ant-design:mobile-outlined", "ant-design:tablet-outlined", "ant-design:red-envelope-outlined",
		"ant-design:book-outlined", "ant-design:file-done-outlined", "ant-design:reconciliation-outlined",
		"ant-design:file-sync-outlined", "ant-design:file-search-outlined", "ant-design:solution-outlined", "ant-design:file-protect-outlined",
		"ant-design:file-add-outlined", "ant-design:file-excel-outlined", "ant-design:file-exclamation-outlined", "ant-design:file-pdf-outlined",
		"ant-design:file-image-outlined", "ant-design:file-markdown-outlined", "ant-design:file-unknown-outlined", "ant-design:file-ppt-outlined",
		"ant-design:file-word-outlined", "ant-design:file-outlined", "ant-design:file-zip-outlined", "ant-design:file-text-outlined", "ant-design:copy-outlined",
		"ant-design:snippets-outlined", "ant-design:audit-outlined", "ant-design:diff-outlined", "ant-design:security-scan-outlined",
		"ant-design:property-safety-outlined", "ant-design:insurance-outlined", "ant-design:alert-outlined", "ant-design:delete-outlined", "ant-design:hourglass-outlined",
		"ant-design:bulb-outlined", "ant-design:experiment-outlined", "ant-design:bell-outlined", "ant-design:trophy-outlined", "ant-design:rest-outlined", "ant-design:usb-outlined",
		"ant-design:skin-outlined", "ant-design:home-outlined", "ant-design:bank-outlined", "ant-design:filter-outlined", "ant-design:funnel-plot-outlined", "ant-design:like-outlined",
		"ant-design:dislike-outlined", "ant-design:unlock-outlined", "ant-design:lock-outlined", "ant-design:customer-service-outlined", "ant-design:flag-outlined",
		"ant-design:money-collect-outlined", "ant-design:medicine-box-outlined", "ant-design:shop-outlined", "ant-design:rocket-outlined", "ant-design:shopping-outlined",
		"ant-design:folder-outlined", "ant-design:folder-open-outlined", "ant-design:folder-add-outlined", "ant-design:deployment-unit-outlined",
		"ant-design:account-book-outlined", "ant-design:contacts-outlined", "ant-design:carry-out-outlined",
		"ant-design:calendar-outlined", "ant-design:scan-outlined", "ant-design:select-outlined", "ant-design:box-plot-outlined", "ant-design:build-outlined", "ant-design:sliders-outlined",
		"ant-design:laptop-outlined", "ant-design:barcode-outlined", "ant-design:camera-outlined", "ant-design:cluster-outlined", "ant-design:gateway-outlined", "ant-design:car-outlined",
		"ant-design:printer-outlined", "ant-design:read-outlined", "ant-design:cloud-server-outlined", "ant-design:cloud-upload-outlined", "ant-design:cloud-outlined",
		"ant-design:cloud-download-outlined", "ant-design:cloud-sync-outlined", "ant-design:notification-outlined", "ant-design:sound-outlined",
		"ant-design:radar-chart-outlined", "ant-design:qrcode-outlined", "ant-design:fund-outlined", "ant-design:mail-outlined", "ant-design:table-outlined",
		"ant-design:idcard-outlined", "ant-design:credit-card-outlined", "ant-design:heart-outlined", "ant-design:block-outlined", "ant-design:star-outlined",
		"ant-design:gold-outlined", "ant-design:heat-map-outlined", "ant-design:wifi-outlined", "ant-design:edit-outlined", "ant-design:key-outlined",
		"ant-design:api-outlined", "ant-design:disconnect-outlined", "ant-design:highlight-outlined", "ant-design:monitor-outlined", "ant-design:link-outlined", "ant-design:man-outlined",
		"ant-design:percentage-outlined", "ant-design:pushpin-outlined", "ant-design:phone-outlined", "ant-design:shake-outlined", "ant-design:tag-outlined",
		"ant-design:tags-outlined", "ant-design:scissor-outlined", "ant-design:share-alt-outlined", "ant-design:branches-outlined", "ant-design:fork-outlined", "ant-design:shrink-outlined",
		"ant-design:vertical-right-outlined", "ant-design:vertical-left-outlined", "ant-design:right-outlined", "ant-design:left-outlined",
		"ant-design:up-outlined", "ant-design:down-outlined", "ant-design:fullscreen-outlined", "ant-design:fullscreen-exit-outlined", "ant-design:double-left-outlined",
		"ant-design:double-right-outlined", "ant-design:arrow-right-outlined", "ant-design:arrow-up-outlined", "ant-design:arrow-left-outlined", "ant-design:arrow-down-outlined",
		"ant-design:upload-outlined", "ant-design:column-height-outlined", "ant-design:vertical-align-bottom-outlined", "ant-design:vertical-align-middle-outlined",
		"ant-design:to-top-outlined", "ant-design:vertical-align-top-outlined", "ant-design:download-outlined", "ant-design:sort-descending-outlined",
		"ant-design:sort-ascending-outlined", "ant-design:fall-outlined", "ant-design:swap-outlined", "ant-design:stock-outlined", "ant-design:rise-outlined",
		"ant-design:menu-outlined", "ant-design:unordered-list-outlined", "ant-design:ordered-list-outlined", "ant-design:align-right-outlined",
		"ant-design:align-center-outlined", "ant-design:align-left-outlined", "ant-design:pic-center-outlined", "ant-design:pic-right-outlined", "ant-design:pic-left-outlined",
		"ant-design:bold-outlined", "ant-design:font-colors-outlined", "ant-design:exclamation-outlined", "ant-design:font-size-outlined", "ant-design:check-circle-outlined",
		"ant-design:info-outlined", "ant-design:ci-outlined", "ant-design:line-height-outlined", "ant-design:dollar-outlined", "ant-design:strikethrough-outlined", "ant-design:compass-outlined",
		"ant-design:underline-outlined", "ant-design:close-circle-outlined", "ant-design:number-outlined", "ant-design:frown-outlined", "ant-design:italic-outlined", "ant-design:info-circle-outlined",
		"ant-design:code-outlined", "ant-design:left-circle-outlined", "ant-design:column-width-outlined", "ant-design:down-circle-outlined", "ant-design:check-outlined",
		"ant-design:ellipsis-outlined", "ant-design:copyright-outlined", "ant-design:dash-outlined", "ant-design:minus-circle-outlined", "ant-design:close-outlined", "ant-design:meh-outlined",
		"ant-design:enter-outlined", "ant-design:plus-circle-outlined", "ant-design:line-outlined", "ant-design:play-circle-outlined", "ant-design:minus-outlined", "ant-design:question-circle-outlined",
		"ant-design:question-outlined", "ant-design:pound-outlined", "ant-design:rollback-outlined", "ant-design:right-circle-outlined", "ant-design:small-dash-outlined", "ant-design:smile-outlined",
		"ant-design:pause-outlined", "ant-design:trademark-outlined", "ant-design:bg-colors-outlined", "ant-design:field-time-outlined", "ant-design:crown-outlined",
		"ant-design:drag-outlined", "ant-design:desktop-outlined", "ant-design:gift-outlined", "ant-design:up-circle-outlined", "ant-design:stop-outlined",
		"ant-design:warning-outlined", "ant-design:fire-outlined", "ant-design:sync-outlined", "ant-design:thunderbolt-outlined", "ant-design:transaction-outlined",
		"ant-design:alipay-outlined", "ant-design:undo-outlined", "ant-design:taobao-outlined", "ant-design:redo-outlined", "ant-design:wechat-outlined", "ant-design:reload-outlined",
		"ant-design:comment-outlined", "ant-design:login-outlined", "ant-design:message-outlined", "ant-design:clear-outlined", "ant-design:dashboard-outlined",
		"ant-design:issues-close-outlined", "ant-design:poweroff-outlined", "ant-design:logout-outlined", "ant-design:pie-chart-outlined", "ant-design:setting-outlined",
		"ant-design:eye-outlined", "ant-design:export-outlined", "ant-design:save-outlined", "ant-design:import-outlined",
		"ant-design:appstore-outlined", "ant-design:close-square-outlined", "ant-design:down-square-outlined", "ant-design:layout-outlined", "ant-design:left-square-outlined",
		"ant-design:play-square-outlined", "ant-design:control-outlined", "ant-design:minus-square-outlined",
		"ant-design:plus-square-outlined", "ant-design:right-square-outlined", "ant-design:project-outlined", "ant-design:wallet-outlined", "ant-design:up-square-outlined",
		"ant-design:calculator-outlined", "ant-design:interaction-outlined", "ant-design:check-square-outlined", "ant-design:border-outlined", "ant-design:border-outer-outlined",
		"ant-design:border-top-outlined", "ant-design:border-bottom-outlined", "ant-design:border-left-outlined", "ant-design:border-right-outlined", "ant-design:border-inner-outlined",
		"ant-design:border-verticle-outlined", "ant-design:border-horizontal-outlined", "ant-design:radius-bottomleft-outlined", "ant-design:radius-bottomright-outlined",
		"ant-design:radius-upleft-outlined", "ant-design:radius-upright-outlined", "ant-design:radius-setting-outlined", "ant-design:user-add-outlined", "ant-design:usergroup-delete-outlined",
		"ant-design:user-delete-outlined", "ant-design:usergroup-add-outlined", "ant-design:user-outlined", "ant-design:area-chart-outlined", "ant-design:line-chart-outlined",
		"ant-design:bar-chart-outlined", "ant-design:container-outlined", "ant-design:safety-certificate-outlined",
        ]
        self.set_width("200px")
        self.set_key()
        return self

    def set_width(self, width: Any):
        """
        设置组件宽度。

        Args:
            width (Any): 宽度值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        style: Dict[str, Any] = {}
        if self.style:
            style = self.style.copy()
        style["width"] = width
        self.style = style
        return self

    def set_options(self, options: List[str]):
        """
        设置可选项数据源。

        Args:
            options (List[str]): 可选项列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.options = options
        return self

    def set_placeholder(self, placeholder: str):
        """
        设置输入框占位文本。

        Args:
            placeholder (str): 占位文本内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placeholder = placeholder
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from typing import Any, List

from tortoise.queryset import QuerySet

from quark import Request


class Action:
    """
    行为类，用于在管理系统中定义按钮、操作等。
    """

    # 设置按钮文字；支持js表达式例如：<%= (status==1 ? '禁用' : '启用') %>，行为在表格行时，可以使用当前行的任意字段值
    name: Any = None

    # 执行成功后刷新的组件
    reload: str = ""

    # 行为接口接收的参数，当行为在表格行展示的时候，可以配置当前行的任意字段
    api_params: List[str] = []

    # 行为接口
    api: str = ""

    # 【必填】这是 action 最核心的配置，来指定该 action 的作用类型，支持：ajax、link、url、drawer、dialog、confirm、cancel、prev、next、copy、close。
    action_type: str = "ajax"

    # 当 action 的作用类型为submit的时候，可以指定提交哪个表格，submitForm为提交表单的key值，为空时提交当前表单
    submit_form: str = ""

    # 配置按钮是否为块级元素
    block: bool = False

    # 批量操作
    batch: bool = False

    # 设置按钮是否为危险按钮
    danger: bool = False

    # 禁用按钮
    disabled: bool = False

    # 配置按钮是否为幽灵按钮
    ghost: bool = False

    # 设置按钮的图标组件
    icon: Any = None

    # 设置按钮类型，primary | ghost | dashed | link | text | default
    type: str = ""

    # 设置按钮大小,large | middle | small | default
    size: str = ""

    # 是否具有loading，当action 的作用类型为ajax,submit时有效
    with_loading: bool = False

    # 行为表单字段
    fields: Any = None

    # 确认标题
    confirm_title: str = ""

    # 确认文字描述
    confirm_text: str = ""

    # 确认类型
    confirm_type: str = ""

    # 只在列表页展示
    only_on_index: bool = False

    # 只在表单页展示
    only_on_form: bool = False

    # 只在详情页展示
    only_on_detail: bool = False

    # 在列表页展示
    show_on_index: bool = False

    # 在列表页行展示
    show_on_index_table_row: bool = False

    # 在表单页展示
    show_on_form: bool = False

    # 在表单页扩展栏展示
    show_on_form_extra: bool = False

    # 在详情页展示
    show_on_detail: bool = False

    # 在详情页扩展栏展示
    show_on_detail_extra: bool = False

    async def handle(self, request: Request, query: QuerySet) -> Any:
        """执行行为句柄"""
        raise NotImplementedError("method not implemented")

    def get_uri_key(self, action: Any) -> str:
        """行为key"""
        class_name = action.__class__.__name__
        return class_name.lower()

    def get_name(self) -> Any:
        """获取名称"""
        return self.name

    def get_reload(self) -> str:
        """执行成功后刷新的组件"""
        return self.reload

    def get_api_params(self) -> List[str]:
        """行为接口接收的参数，当行为在表格行展示的时候，可以配置当前行的任意字段"""
        return self.api_params

    def get_api(self) -> str:
        """执行行为的接口"""
        return self.api

    def get_action_type(self) -> str:
        """【必填】这是 action 最核心的配置，来指定该 action 的作用类型"""
        return self.action_type

    def get_submit_form(self) -> str:
        """当 action 的作用类型为submit的时候，可以指定提交哪个表格"""
        return self.submit_form

    def get_type(self) -> str:
        """设置按钮类型，primary | ghost | dashed | link | text | default"""
        return self.type

    def get_size(self) -> str:
        """设置按钮大小,large | middle | small | default"""
        return self.size

    def get_with_loading(self) -> bool:
        """是否具有loading，当action 的作用类型为ajax,submit时有效"""
        return self.with_loading

    def get_icon(self) -> Any:
        """设置按钮的图标组件"""
        return self.icon

    def get_block(self) -> bool:
        """配置按钮是否为块级元素"""
        return self.block

    def get_batch(self) -> bool:
        """批量操作"""
        return self.batch

    def get_danger(self) -> bool:
        """设置按钮是否为危险按钮"""
        return self.danger

    def get_disabled(self) -> bool:
        """禁用按钮"""
        return self.disabled

    def get_ghost(self) -> bool:
        """配置按钮是否为幽灵按钮"""
        return self.ghost

    def get_fields(self) -> Any:
        """行为表单字段"""
        return self.fields

    def get_confirm_title(self) -> str:
        """确认标题"""
        return self.confirm_title

    def get_confirm_text(self) -> str:
        """确认文字"""
        return self.confirm_text

    def get_confirm_type(self) -> str:
        """确认类型"""
        return self.confirm_type

    def set_name(self, name: str):
        """设置名称"""
        self.name = name

    def set_reload(self, component_key: str):
        """设置执行成功后刷新的组件"""
        self.reload = component_key

    def set_api_params(self, api_params: List[str]):
        """行为接口接收的参数，当行为在表格行展示的时候，可以配置当前行的任意字段"""
        self.api_params = api_params

    def set_api(self, api: str):
        """执行行为的接口"""
        self.api = api

    def set_action_type(self, action_type: str):
        """【必填】这是 action 最核心的配置，来指定该 action 的作用类型"""
        self.action_type = action_type

    def set_submit_form(self, submit_form: str):
        """当 action 的作用类型为submit的时候，可以指定提交哪个表格"""
        self.submit_form = submit_form

    def set_type(self, button_type: str):
        """设置按钮类型，primary | ghost | dashed | link | text | default"""
        self.type = button_type

    def set_size(self, size: str):
        """设置按钮大小,large | middle | small | default"""
        self.size = size

    def set_with_loading(self, loading: bool):
        """是否具有loading，当action 的作用类型为ajax,submit时有效"""
        self.with_loading = loading

    def set_icon(self, icon: str):
        """设置按钮的图标组件"""
        self.icon = icon

    def set_block(self, block: bool):
        """配置按钮是否为块级元素"""
        self.block = block

    def set_batch(self, batch: bool):
        """批量操作"""
        self.batch = batch

    def set_danger(self, danger: bool):
        """设置按钮是否为危险按钮"""
        self.danger = danger

    def set_disabled(self, disabled: bool):
        """禁用按钮"""
        self.disabled = disabled

    def set_ghost(self, ghost: bool):
        """配置按钮是否为幽灵按钮"""
        self.ghost = ghost

    def set_fields(self, fields: Any):
        """行为表单字段"""
        self.fields = fields

    def set_confirm_title(self, confirm_title: str):
        """确认标题"""
        self.confirm_title = confirm_title

    def set_confirm_text(self, confirm_text: str):
        """确认文字"""
        self.confirm_text = confirm_text

    def set_confirm_type(self, confirm_type: str):
        """确认类型"""
        self.confirm_type = confirm_type

    def with_confirm(self, title: str, text: str, confirm_type: str):
        """设置行为前的确认操作"""
        self.confirm_title = title
        self.confirm_text = text
        self.confirm_type = confirm_type

    def set_only_on_index(self, value: bool):
        """只在列表页展示"""
        self.only_on_index = value
        self.show_on_index = value
        self.show_on_detail = not value
        self.show_on_index_table_row = not value
        self.show_on_form = not value
        self.show_on_form_extra = not value
        self.show_on_detail_extra = not value

    def set_except_on_index(self):
        """除了列表页外展示"""
        self.show_on_detail = True
        self.show_on_index_table_row = True
        self.show_on_form = True
        self.show_on_form_extra = True
        self.show_on_detail_extra = True
        self.show_on_index = False

    def set_only_on_form(self, value: bool):
        """只在表单页展示"""
        self.show_on_form = value
        self.show_on_index = not value
        self.show_on_detail = not value
        self.show_on_index_table_row = not value
        self.show_on_form_extra = not value
        self.show_on_detail_extra = not value

    def set_except_on_form(self):
        """除了表单页外展示"""
        self.show_on_index = True
        self.show_on_detail = True
        self.show_on_index_table_row = True
        self.show_on_form = False
        self.show_on_form_extra = True
        self.show_on_detail_extra = True

    def set_only_on_form_extra(self, value: bool):
        """只在表单页右上角自定义区域展示"""
        self.show_on_form = not value
        self.show_on_index = not value
        self.show_on_detail = not value
        self.show_on_index_table_row = not value
        self.show_on_form_extra = value
        self.show_on_detail_extra = not value

    def set_except_on_form_extra(self):
        """除了表单页右上角自定义区域外展示"""
        self.show_on_index = True
        self.show_on_detail = True
        self.show_on_index_table_row = True
        self.show_on_form = True
        self.show_on_form_extra = False
        self.show_on_detail_extra = True

    def set_only_on_detail(self, value: bool):
        """只在详情页展示"""
        self.only_on_detail = value
        self.show_on_detail = value
        self.show_on_index = not value
        self.show_on_index_table_row = not value
        self.show_on_form = not value
        self.show_on_form_extra = not value
        self.show_on_detail_extra = not value

    def set_except_on_detail(self):
        """除了详情页外展示"""
        self.show_on_index = True
        self.show_on_detail = False
        self.show_on_index_table_row = True
        self.show_on_form = True
        self.show_on_form_extra = True
        self.show_on_detail_extra = True

    def set_only_on_detail_extra(self, value: bool):
        """只在详情页右上角自定义区域展示"""
        self.show_on_form = not value
        self.show_on_index = not value
        self.show_on_detail = not value
        self.show_on_index_table_row = not value
        self.show_on_form_extra = not value
        self.show_on_detail_extra = value

    def set_except_on_detail_extra(self):
        """除了详情页右上角自定义区域外展示"""
        self.show_on_index = True
        self.show_on_detail = True
        self.show_on_index_table_row = True
        self.show_on_form = True
        self.show_on_form_extra = True
        self.show_on_detail_extra = False

    def set_only_on_index_table_row(self, value: bool):
        """在表格行内展示"""
        self.show_on_index_table_row = value
        self.show_on_index = not value
        self.show_on_detail = not value
        self.show_on_form = not value
        self.show_on_form_extra = not value
        self.show_on_detail_extra = not value

    def set_except_on_index_table_row(self):
        """除了表格行内外展示"""
        self.show_on_index_table_row = False
        self.show_on_index = True
        self.show_on_detail = True
        self.show_on_form = True
        self.show_on_form_extra = True
        self.show_on_detail_extra = True

    def set_show_on_index(self):
        """在列表页展示"""
        self.show_on_index = True

    def set_show_on_form(self):
        """在表单页展示"""
        self.show_on_form = True

    def set_show_on_form_extra(self):
        """在表单页右上角自定义区域展示"""
        self.show_on_form_extra = True

    def set_show_on_detail(self):
        """在详情页展示"""
        self.show_on_detail = True

    def set_show_on_detail_extra(self):
        """在详情页右上角自定义区域展示"""
        self.show_on_detail_extra = True

    def set_show_on_index_table_row(self):
        """在表格行内展示"""
        self.show_on_index_table_row = True

    def shown_on_index(self) -> bool:
        """判断是否在列表页展示"""
        if self.only_on_index:
            return True
        if self.only_on_detail:
            return False
        if self.only_on_form:
            return False
        return self.show_on_index

    def shown_on_form(self) -> bool:
        """判断是否在表单页展示"""
        if self.only_on_form:
            return True
        if self.only_on_detail:
            return False
        if self.only_on_index:
            return False
        return self.show_on_form

    def shown_on_detail(self) -> bool:
        """判断是否在详情页展示"""
        if self.only_on_detail:
            return True
        if self.only_on_index:
            return False
        if self.only_on_form:
            return False
        return self.show_on_detail

    def shown_on_index_table_row(self) -> bool:
        """判断是否在表格行内展示"""
        return self.show_on_index_table_row

    def shown_on_form_extra(self) -> bool:
        """判断是否在表单页右上角自定义区域展示"""
        return self.show_on_form_extra

    def shown_on_detail_extra(self) -> bool:
        """判断是否在详情页右上角自定义区域展示"""
        return self.show_on_detail_extra



from quark import APIRouter, JSONResponse, Request, jsonable_encoder

from .. import loader

router = APIRouter(prefix="/api/admin", tags=["管理员登录"])


@router.get("/auth/{resource}/index")
async def index(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Auth")
    return JSONResponse(
        content=jsonable_encoder(await res.render(request), exclude_none=True)
    )

@router.get("/auth/{resource}/captcha")
async def captcha(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Auth")
    return await res.captcha(request)


@router.post("/auth/{resource}/login")
async def login(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Auth")
    return JSONResponse(
        content=jsonable_encoder(await res.login(request), exclude_none=True)
    )


@router.get("/auth/{resource}/userInfo")
async def user_info(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Auth")
    return JSONResponse(
        content=jsonable_encoder(await res.user_info(request), exclude_none=True)
    )

@router.get("/auth/{resource}/userRoutes")
async def user_routes(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Auth")
    return JSONResponse(
        content=jsonable_encoder(await res.user_routes(request), exclude_none=True)
    )

@router.get("/auth/{resource}/logout")
async def logout(request: Request, resource: str):
    res = await loader.load_resource_object(request, resource, "Auth")
    return JSONResponse(
        content=jsonable_encoder(await res.logout(request), exclude_none=True)
    )



from typing import Any, Dict, List, Optional, Union

from pydantic import Field, model_validator

from ..component import Component


class Column(Component):
    component: Optional[str] = "column"
    title: Optional[str] = None
    attribute: Optional[str] = None
    align: str = "left"
    data_index: Optional[str] = None
    fixed: Any = None
    tooltip: Optional[str] = None
    ellipsis: Optional[bool] = None
    copyable: Optional[bool] = None
    value_enum: Optional[Union[Dict[Any, Any], List[Dict[str, Any]]]] = None
    value_type: Optional[str] = None
    hide_in_search: bool = True
    hide_in_table: Optional[bool] = None
    hide_in_form: Optional[bool] = None
    filters: Optional[Union[bool, Dict[str, str], List[Dict[str, str]]]] = None
    order: Optional[int] = None
    sorter: Any = None
    span: Optional[int] = None
    width: Optional[int] = None
    editable: Optional[Dict[str, Any]] = None
    actions: Optional[Any] = None
    form_item_props: Optional[Any] = None
    field_props: Optional[Any] = None
    style: Optional[Dict[str, Any]] = None

    @model_validator(mode="after")
    def init(self):
        self.set_key()
        return self

    # 设置方法（链式调用）
    def set_style(self, style: Dict[str, Any]):
        self.style = style
        return self

    def set_title(self, title: str):
        self.title = title
        return self

    def set_attribute(self, attribute: str):
        self.componentkey = attribute
        self.data_index = attribute
        self.attribute = attribute
        return self

    def set_align(self, align: str):
        self.align = align
        return self

    def set_fixed(self, fixed: Any):
        self.fixed = fixed
        return self

    def set_tooltip(self, tooltip: str):
        self.tooltip = tooltip
        return self

    def set_ellipsis(self, ellipsis: bool):
        self.ellipsis = ellipsis
        return self

    def set_copyable(self, copyable: bool):
        self.copyable = copyable
        return self

    def set_value_enum(self, value_enum: Union[Dict[Any, Any], List[Dict[str, Any]]]):
        if isinstance(value_enum, dict):
            value_enum_str = {}
            value_enum_int = {}
            for k, v in value_enum.items():
                if isinstance(k, str):
                    value_enum_str[k] = v
                else:
                    value_enum_int[int(k)] = v
            self.value_enum = value_enum_str or value_enum_int
        else:
            self.value_enum = value_enum
        return self

    def set_value_type(self, value_type: str):
        self.value_type = value_type
        return self

    def set_hide_in_search(self, hide: bool):
        self.hide_in_search = hide
        return self

    def set_hide_in_table(self, hide: bool):
        self.hide_in_table = hide
        return self

    def set_filters(self, filters: Union[bool, Dict[str, str], List[Dict[str, str]]]):
        if isinstance(filters, dict):
            self.filters = [{"text": v, "value": k} for k, v in filters.items()]
        else:
            self.filters = filters
        return self

    def set_order(self, order: int):
        self.order = order
        return self

    def set_sorter(self, sorter: Any):
        self.sorter = sorter
        return self

    def set_span(self, span: int):
        self.span = span
        return self

    def set_width(self, width: Optional[int]):
        self.width = width
        return self

    def set_editable(self, name: str, options: Any, action: str):
        get_options = []
        if name == "select" and isinstance(options, list):
            for idx, v in enumerate(options):
                get_options.append({"label": v, "value": idx})
        else:
            get_options = options

        self.editable = {
            "name": name,
            "options": get_options,
            "action": action,
        }
        return self

    def set_actions(self, actions: Any):
        self.actions = actions
        return self

    def set_form_item_props(self, form_item_props: Any):
        self.form_item_props = form_item_props
        return self

    def set_field_props(self, field_props: Any):
        self.field_props = field_props
        return self



from typing import Any, Optional
from .base import Base


class Group(Base):

    component: str = "groupField"
    """
    组件名称
    """

    title: Optional[str] = None
    """
    分组标题
    """

    body: Optional[Any] = None
    """
    组件内容
    """

    size: int = 32
    """
    子元素个数，默认 32
    """

    def set_title(self, title: str):
        """
        设置分组标题。

        Args:
            title (str): 分组标题内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.title = title
        return self

    def set_body(self, body: Any):
        """
        设置组件内容。

        Args:
            body (Any): 组件内容。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.body = body
        return self

    def set_size(self, size: int):
        """
        设置子元素个数。

        Args:
            size (int): 子元素个数。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self



import json
from typing import Any, Dict, List, Optional

from pydantic import model_validator

from ..component import Component


class Form(Component):
    """
    表单组件类
    """

    title: Optional[str] = None
    width: Optional[str] = None
    colon: bool = True
    values: Dict[str, Any] = {}
    initial_values: Dict[str, Any] = {}
    label_align: str = "right"
    name: Optional[str] = None
    preserve: bool = True
    required_mark: bool = True
    scroll_to_first_error: bool = False
    size: str = "default"
    date_formatter: str = "string"
    layout: str = "horizontal"
    grid: bool = False
    row_props: Optional[Dict[str, Any]] = None
    label_col: Optional[Dict[str, Any]] = {"span": 4}
    wrapper_col: Optional[Dict[str, Any]] = {"span": 20}
    button_wrapper_col: Optional[Dict[str, Any]] = {"offset": 4, "span": 20}
    api: Optional[str] = None
    api_type: str = "POST"
    target_blank: bool = False
    init_api: Optional[str] = None
    body: Any = None
    actions: Optional[List[Any]] = None
    component_key: Optional[str] = None
    style: Optional[dict] = None

    @model_validator(mode="after")
    def init(self) -> "Form":
        """
        初始化
        """
        self.component = "form"
        self.colon = True
        self.label_align = "right"
        self.preserve = True
        self.required_mark = True
        self.size = "default"
        self.date_formatter = "string"
        self.layout = "horizontal"
        self.label_col = {
            "span": 4,
        }
        self.wrapper_col = {
            "span": 20,
        }
        self.button_wrapper_col = {
            "offset": 4,
            "span": 20,
        }
        self.api_type = "POST"

        self.set_key()

        return self

    def set_style(self, style: Dict[str, Any]) -> "Form":
        """
        Set style.
        """
        self.style = style
        return self

    def set_title(self, title: str) -> "Form":
        """
        配置表单标题
        """
        self.title = title
        return self

    def set_width(self, width: str) -> "Form":
        """
        配置表单宽度
        """
        self.width = width
        return self

    def set_colon(self, colon: bool) -> "Form":
        """
        配置 Form.Item 的 colon 的默认值。表示是否显示 label 后面的冒号 (只有在属性 layout 为 horizontal 时有效)
        """
        self.colon = colon
        return self

    def parse_initial_value(self, item: Any, initial_values: Dict[str, Any]) -> Any:
        """
        解析initialValue
        """
        value = None

        # 数组直接返回
        if isinstance(item, list):
            return None

        # 这里假设item是一个对象，具有name、default_value和value属性
        name = getattr(item, "name", "")
        if not name:
            return None

        if hasattr(item, "default_value") and item.default_value is not None:
            value = item.default_value

        if hasattr(item, "value") and item.value is not None:
            value = item.value

        if name in initial_values and initial_values[name] is not None:
            value = initial_values[name]

        return value

    def find_fields(self, fields: Any, when: bool) -> Any:
        """
        查找字段
        """
        items = []

        if isinstance(fields, list):
            for v in fields:
                items.extend(self.field_parser(v, when))
        else:
            items.extend(self.field_parser(fields, when))

        return items

    def field_parser(self, v: Any, when: bool) -> List[Any]:
        """
        解析字段
        """
        items = []

        # 数组直接返回
        if isinstance(v, list):
            return items

        # 检查是否有body属性
        if hasattr(v, "body") and v.body is not None:
            get_items = self.find_fields(v.body, True)
            if isinstance(get_items, list) and len(get_items) > 0:
                items.extend(get_items)
            return items

        # 检查是否有tab_panes属性
        if hasattr(v, "tab_panes") and v.tab_panes is not None:
            get_items = self.find_fields(v.tab_panes, True)
            if isinstance(get_items, list) and len(get_items) > 0:
                items.extend(get_items)
            return items

        # 默认情况
        component = getattr(v, "component", "")
        if "Field" in component:
            items.append(v)
            if when:
                when_fields = self.get_when_fields(v)
                if len(when_fields) > 0:
                    items.extend(when_fields)

        return items

    def get_when_fields(self, item: Any) -> List[Any]:
        """
        获取When组件中的字段
        """
        items = []

        if not hasattr(item, "when") or item.when is None:
            return items

        get_when = item.get_when() if hasattr(item, "get_when") else None

        if get_when is None:
            return items

        when_items = getattr(get_when, "items", None)
        if when_items is None:
            return items

        for v in when_items:
            if v.body is not None:
                if isinstance(v.body, list):
                    if len(v.body) > 0:
                        items.extend(v.body)
                else:
                    items.append(v.body)

        return items

    def set_initial_values(self, initial_values: Dict[str, Any]) -> "Form":
        """
        表单默认值，只有初始化以及重置时生效
        """
        data = initial_values.copy()

        fields = self.find_fields(self.body, True)
        if isinstance(fields, list):
            for v in fields:
                value = self.parse_initial_value(v, initial_values)
                if value is not None:
                    name = getattr(v, "name", "")
                    data[name] = value

        for k, v in data.items():
            if isinstance(v, str) and "[" in v:
                try:
                    m = json.loads(v)
                    if isinstance(m, list):
                        data[k] = m
                    elif isinstance(m, dict) and "{" in v:
                        data[k] = m
                except json.JSONDecodeError:
                    if "{" in v:
                        try:
                            m = json.loads(v)
                            if isinstance(m, dict):
                                data[k] = m
                        except json.JSONDecodeError:
                            pass
            elif isinstance(v, str) and "{" in v:
                try:
                    m = json.loads(v)
                    if isinstance(m, dict):
                        data[k] = m
                except json.JSONDecodeError:
                    pass

        self.initial_values = data
        return self

    def set_label_align(self, label_align: str) -> "Form":
        """
        label 标签的文本对齐方式,left | right
        """
        self.label_align = label_align
        return self

    def set_name(self, name: str) -> "Form":
        """
        表单名称，会作为表单字段 id 前缀使用
        """
        self.name = name
        return self

    def set_preserve(self, preserve: bool) -> "Form":
        """
        当字段被删除时保留字段值
        """
        self.preserve = preserve
        return self

    def set_required_mark(self, required_mark: bool) -> "Form":
        """
        必选样式，可以切换为必选或者可选展示样式。此为 Form 配置，Form.Item 无法单独配置
        """
        self.required_mark = required_mark
        return self

    def set_scroll_to_first_error(self, scroll_to_first_error: bool) -> "Form":
        """
        提交失败自动滚动到第一个错误字段
        """
        self.scroll_to_first_error = scroll_to_first_error
        return self

    def set_size(self, size: str) -> "Form":
        """
        设置字段组件的尺寸（仅限 antd 组件）,small | middle | large
        """
        self.size = size
        return self

    def set_date_formatter(self, date_formatter: str) -> "Form":
        """
        自动格式数据，例如 moment 的表单,支持 string 和 number 两种模式
        """
        self.date_formatter = date_formatter
        return self

    def set_layout(self, layout: str) -> "Form":
        """
        表单布局，horizontal | vertical
        """
        if layout == "vertical":
            self.label_col = None
            self.wrapper_col = None
            self.button_wrapper_col = None

        self.layout = layout
        return self

    def set_grid(self, grid: bool) -> "Form":
        """
        开启栅格化模式，宽度默认百分比，请使用 colProps 控制宽度，示例：https://procomponents.ant.design/components/form#栅格化布局
        """
        self.grid = grid
        return self

    def set_row_props(self, row_props: Dict[str, Any]) -> "Form":
        """
        开启 grid 模式时传递给 Row, 仅在ProFormGroup, ProFormList, ProFormFieldSet 中有效，默认：{ gutter: 8 }
        """
        self.row_props = row_props
        return self

    def set_label_col(self, label_col: Dict[str, Any]) -> "Form":
        """
        label 标签布局，同 <Col> 组件，设置 span offset 值，如 {span: 3, offset: 12} 或 sm: {span: 3, offset: 12}
        """
        self.label_col = label_col
        return self

    def set_wrapper_col(self, wrapper_col: Dict[str, Any]) -> "Form":
        """
        需要为输入控件设置布局样式时，使用该属性，用法同 labelCol
        """
        if self.layout == "vertical":
            raise Exception("If layout set vertical mode,can't set wrapperCol!")

        self.wrapper_col = wrapper_col
        return self

    def set_button_wrapper_col(self, button_wrapper_col: Dict[str, Any]) -> "Form":
        """
        表单按钮布局样式,默认：['offset' => 2, 'span' => 22 ]
        """
        if self.layout == "vertical":
            raise Exception("If layout set vertical mode,can't set buttonWrapperCol!")

        self.button_wrapper_col = button_wrapper_col
        return self

    def set_api(self, api: str) -> "Form":
        """
        表单提交的接口链接
        """
        self.api = api
        return self

    def set_api_type(self, api_type: str) -> "Form":
        """
        表单提交接口的类型
        """
        self.api_type = api_type
        return self

    def set_target_blank(self, target_blank: bool) -> "Form":
        """
        提交表单的数据是否打开新页面，只有在GET类型的时候有效
        """
        self.target_blank = target_blank
        return self

    def set_init_api(self, init_api: str) -> "Form":
        """
        获取表单数据
        """
        self.init_api = init_api
        return self

    def set_body(self, items: Any) -> "Form":
        """
        表单项
        """
        self.body = items
        return self

    def parse_submit_data(self, data: Dict[str, Any]) -> Dict[str, Any]:
        """
        解析保存提交数据库前的值
        """
        # 注意：这里假设self.body是一个列表
        if isinstance(self.body, list):
            for v in self.body:
                ignore = getattr(v, "ignore", False)

                # 删除忽略的值
                if ignore:
                    name = getattr(v, "name", "")
                    if name and name in data:
                        del data[name]

        for k, v in data.items():
            if isinstance(v, dict):
                data[k] = json.dumps(v)

        return data

    def set_actions(self, actions: List[Any]) -> "Form":
        """
        设置表单行为
        """
        self.actions = actions
        return self



from typing import List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.template.action import Action


class DeleteSpecial(Action):

    def __init__(self, name=None):
        # 名称支持动态表达式，Python中可用字符串或模板渲染实现
        self.name = name or "<%= (id==1 ? '' : '删除') %>"
        self.type = "link"
        self.size = "small"
        self.reload = "table"
        self.with_confirm(
            "确定要删除吗？", "删除后数据将无法恢复，请谨慎操作！", "modal"
        )
        self.set_only_on_index_table_row(True)
        self.set_api_params(["id"])

    def get_api_params(self) -> List[str]:
        return ["id"]

    async def handle(self, request: Request, query: QuerySet):
        try:
            await query.delete()
            return Message.success("操作成功")
        except Exception as e:
            return Message.error(str(e))



import json
import os
import random
import string

import pandas as pd
from openpyxl import Workbook
from openpyxl.styles import Font
from tortoise.models import Model

from quark import Message, Request


class ImportRequest:

    # 请求对象
    request: Request = None

    # 查询对象
    query: Model = None

    # 列表页字段
    fields: list = None

    def __init__(
        self,
        request: Request,
        query: Model,
        fields: list,
    ):
        self.request = request
        self.query = query
        self.fields = fields

    def handle(self, index_route):
        """
        处理导入请求
        """
        body_bytes = self.request.body()
        data = json.loads(body_bytes)
        file_ids = data.get("fileId", [])

        if not file_ids or not isinstance(file_ids, list) or len(file_ids) == 0:
            return Message.error("参数错误")

        file_id_info = file_ids[0]
        file_id = file_id_info.get("id")
        if not file_id:
            return Message.error("参数错误")

        # 获取Excel数据
        import_data = self.get_excel_data(file_id)
        if not import_data:
            return Message.error("文件数据为空")

        # 表头与数据分离
        import_head = import_data[0]
        import_data = import_data[1:]

        # 导入前回调
        lists = template.before_importing(import_data)

        import_result = True
        total_num = len(lists)
        success_num = 0
        failed_num = 0
        failed_data = []

        fields = self.fields

        for item in lists:
            form_values = self.transform_form_values(fields, item)

            # 验证器
            validator = template.validator_for_import(form_values)
            if validator:
                import_result = False
                failed_num += 1
                padding_len = len(fields) - len(item)
                item += [None] * padding_len
                item.append(validator.error())
                failed_data.append(item)
                continue

            # BeforeSaving 回调
            try:
                submit_data, err = template.before_saving(form_values)
                if err:
                    raise Exception(err)
            except Exception as e:
                import_result = False
                failed_num += 1
                padding_len = len(fields) - len(item)
                item += [None] * padding_len
                item.append(str(e))
                failed_data.append(item)
                continue

            # 提交数据
            data_to_submit = self.get_submit_data(fields, submit_data)
            try:
                record = YourModel(**data_to_submit)
                db.session.add(record)
                db.session.commit()

                # AfterImported 回调
                last_id = record.id
                after_err = template.after_imported(last_id, data_to_submit)
                if after_err:
                    raise Exception(after_err)

                success_num += 1
            except Exception as e:
                db.session.rollback()
                import_result = False
                failed_num += 1
                padding_len = len(fields) - len(item)
                item += [None] * padding_len
                item.append(str(e))
                failed_data.append(item)

        # 返回失败数据
        if not import_result:
            static_path = app.config["STATIC_PATH"]
            storage_dir = os.path.join(static_path, "app", "storage", "failImports")
            file_name = self.generate_random_filename(40) + ".xlsx"
            file_url = f"//{request.host}/storage/failImports/{file_name}"

            if not os.path.exists(storage_dir):
                os.makedirs(storage_dir)

            wb = Workbook()
            ws = wb.active
            ws.title = "Sheet1"

            # 写入表头
            import_head.append("错误信息")
            for col_idx, title in enumerate(import_head, start=1):
                ws[f"{generate_column_label(col_idx)}1"] = title

            # 写入失败数据
            for row_idx, row in enumerate(failed_data, start=2):
                for col_idx, cell_value in enumerate(row, start=1):
                    ws[f"{generate_column_label(col_idx)}{row_idx}"] = cell_value
                    if col_idx == len(row):
                        cell = ws[f"{generate_column_label(col_idx)}{row_idx}"]
                        cell.font = Font(color="FF0000")

            # 保存文件
            file_path = os.path.join(storage_dir, file_name)
            wb.save(file_path)

            response_html = (
                f"<div style='margin: 20px;'>"
                f"<p>导入总量: {total_num}</p>"
                f"<p>成功数量: {success_num}</p>"
                f"<p>失败数量: <span style='color:#ff4d4f'>{failed_num}</span> "
                f"<a href='{file_url}' target='_blank'>下载失败数据</a></p>"
                f"</div>"
            )

            return jsonify({"html": response_html}), 200

        return redirect(
            f"/layout/index?api={index_route.replace(':resource', request.view_args.get('resource'))}"
        )

    def get_excel_data(self, file_id):
        """
        获取附件服务中的 Excel 数据（需根据你的业务系统实现）
        """
        from your_app.models import Attachment

        attachment = Attachment.query.get(file_id)
        if not attachment:
            return []
        file_path = attachment.file_path
        if not os.path.exists(file_path):
            return []
        # 使用 openpyxl 或 pandas 读取 Excel 数据

        df = pd.read_excel(file_path, header=None)
        return df.values.tolist()

    def transform_form_values(self, fields, data):
        result = {}
        for idx, field in enumerate(fields):
            name = field.get("name")
            if idx < len(data):
                result[name] = data[idx]
        return result

    def get_submit_data(self, fields, submit_data):
        result = {}

        for field in fields:
            component = field.get("component")
            name = field.get("name")
            value = submit_data.get(name)

            if component == "inputNumberField":
                result[name] = value
            elif component == "textField":
                result[name] = str(value).strip("\n") if value else None
            elif component in ["selectField", "checkboxField", "radioField"]:
                label = value
                option_value = field.get("getOptionValue")(label)
                result[name] = option_value
            elif component == "switchField":
                label = value
                option_value = field.get("getOptionValue")(label)
                result[name] = option_value
            else:
                result[name] = value

            # 转换复杂类型为 JSON 字符串
            if isinstance(result[name], (list, dict)):
                result[name] = json.dumps(result[name])

        return result

    def generate_random_filename(self, length=40):
        chars = string.ascii_letters + string.digits
        return "".join(random.choice(chars) for _ in range(length))



from typing import Any, List

from quark import Request, Resource, models, services
from quark.app import actions, searches
from quark.component.form import field
from quark.component.form.rule import Rule
from quark.component.table.table import Expandable


class Department(Resource):
    """
    部门管理
    """

    async def init(self, request: Request):

        self.table.set_expandable(Expandable(default_expanded_row_keys=[1]))

        # 页面标题
        self.title = "部门"

        # 模型
        self.model = models.Department

        # 树表
        self.table_list_to_tree = True

        # 排序字段
        self.index_query_order = ["sort", "id"]

        # 分页
        self.page_size = False

        return self

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        departments = await services.DepartmentService().get_list()

        return [
            field.hidden("id", "ID"),
            field.hidden("pid", "父级ID").only_on_index(),
            field.text("name", "名称").set_rules(
                [
                    Rule.required("部门名称必须填写"),
                    Rule.min(2, "部门名称不能少于2个字符"),
                    Rule.max(100, "部门名称不能超过100个字符"),
                ]
            ),
            field.dependency().set_when(
                "id",
                "!=",
                1,
                lambda: (
                    field.tree_select("pid", "父节点")
                    .set_tree_data(departments, "pid", "name", "id")
                    .set_rules([Rule.required("请选择父节点")])
                    .set_default_value(1)
                    .only_on_forms()
                ),
            ),
            field.number("sort", "排序").set_editable(True).set_default_value(0),
            field.switch("status", "状态")
            .set_editable(True)
            .set_true_value("正常")
            .set_false_value("禁用")
            .set_default_value(True),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("name", "名称"),
            searches.Status(),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.CreateModal(self),
            actions.BatchDelete(),
            actions.EditModal(self),
            actions.DeleteSpecial(),
        ]



from typing import Dict, List, Optional, Any
from .base import Base


class TimeRange(Base):

    component: str = "timeRangeField"
    """
    组件名称
    """

    allow_clear: bool = True
    """
    是否支持清除，默认值为 True
    """

    auto_focus: bool = False
    """
    自动获取焦点，默认值为 False
    """

    bordered: bool = True
    """
    是否有边框，默认值为 True
    """

    class_name: Optional[str] = None
    """
    自定义类名，默认值为 None
    """

    clear_icon: Optional[Any] = None
    """
    自定义的清除图标，默认值为 None
    """

    clear_text: Optional[str] = None
    """
    清除按钮的提示文案，默认值为 None
    """

    default_value: Optional[Any] = None
    """
    默认的选中项，默认值为 None
    """

    disabled: Optional[Any] = None
    """
    禁用，默认值为 None
    """

    disabled_time: Optional[Any] = None
    """
    不可选择的时间，默认值为 None
    """

    format: Optional[str] = "HH:mm"
    """
    设置日期格式，为数组时支持多格式匹配，展示以第一个为准，默认值为 "HH:mm"
    """

    hide_disabled_options: bool = False
    """
    隐藏禁止选择的选项，默认值为 False
    """

    hour_step: int = 0
    """
    小时选项间隔，默认值为 0
    """

    input_read_only: bool = False
    """
    设置输入框为只读（避免在移动设备上打开虚拟键盘），默认值为 False
    """

    minute_step: int = 0
    """
    分钟选项间隔，默认值为 0
    """

    open: bool = False
    """
    控制浮层显隐，默认值为 False
    """

    placeholder: List[str] = ["开始时间", "结束时间"]
    """
    输入框占位文本，默认值为 ["开始时间", "结束时间"]
    """

    placement: Optional[str] = None
    """
    浮层预设位置，bottomLeft bottomRight topLeft topRight，默认值为 None
    """

    popup_class_name: Optional[str] = None
    """
    额外的弹出日历 className，默认值为 None
    """

    popup_style: Optional[Any] = None
    """
    额外的弹出日历样式，默认值为 None
    """

    second_step: int = 0
    """
    秒选项间隔，默认值为 0
    """

    show_now: bool = False
    """
    面板是否显示“此刻”按钮，默认值为 False
    """

    size: Optional[str] = None
    """
    输入框大小，large | middle | small，默认值为 None
    """

    status: Optional[str] = None
    """
    设置校验状态，'error' | 'warning'，默认值为 None
    """

    suffix_icon: Optional[Any] = None
    """
    自定义的选择框后缀图标，默认值为 None
    """

    use_12_hours: bool = False
    """
    使用 12 小时制，为 True 时 format 默认为 h:mm:ss a，默认值为 False
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式，默认值为 None
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    def set_allow_clear(self, allow_clear: bool):
        """
        设置是否可以点击清除图标删除内容。

        Args:
            allow_clear (bool): 是否允许清除。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.allow_clear = allow_clear
        return self

    def set_auto_focus(self, auto_focus: bool):
        """
        设置自动获取焦点。

        Args:
            auto_focus (bool): 是否自动获取焦点。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.auto_focus = auto_focus
        return self

    def set_bordered(self, bordered: bool):
        """
        设置是否有边框。

        Args:
            bordered (bool): 是否有边框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.bordered = bordered
        return self

    def set_class_name(self, class_name: str):
        """
        设置自定义类名。

        Args:
            class_name (str): 自定义类名。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.class_name = class_name
        return self

    def set_clear_icon(self, clear_icon: Any):
        """
        设置自定义的清除图标。

        Args:
            clear_icon (Any): 清除图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.clear_icon = clear_icon
        return self

    def set_clear_text(self, clear_text: str):
        """
        设置清除按钮的提示文案。

        Args:
            clear_text (str): 提示文案。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.clear_text = clear_text
        return self

    def set_disabled_time(self, disabled_time: Any):
        """
        设置不可选择的时间。

        Args:
            disabled_time (Any): 不可选择的时间。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.disabled_time = disabled_time
        return self

    def set_hide_disabled_options(self, hide_disabled_options: bool):
        """
        设置隐藏禁止选择的选项。

        Args:
            hide_disabled_options (bool): 是否隐藏禁止选择的选项。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.hide_disabled_options = hide_disabled_options
        return self

    def set_hour_step(self, hour_step: int):
        """
        设置小时选项间隔。

        Args:
            hour_step (int): 小时选项间隔。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.hour_step = hour_step
        return self

    def set_minute_step(self, minute_step: int):
        """
        设置分钟选项间隔。

        Args:
            minute_step (int): 分钟选项间隔。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.minute_step = minute_step
        return self

    def set_second_step(self, second_step: int):
        """
        设置秒选项间隔。

        Args:
            second_step (int): 秒选项间隔。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.second_step = second_step
        return self

    def set_use_12_hours(self, use_12_hours: bool):
        """
        设置使用 12 小时制。

        Args:
            use_12_hours (bool): 是否使用 12 小时制。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.use_12_hours = use_12_hours
        if use_12_hours and self.format == "HH:mm":
            self.format = "h:mm:ss a"
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_format(self, format_str: str):
        """
        设置日期格式。

        Args:
            format_str (str): 日期格式字符串。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.format = format_str
        return self

    def set_popup_class_name(self, popup_class_name: str):
        """
        设置额外的弹出日历 className。

        Args:
            popup_class_name (str): className 字符串。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.popup_class_name = popup_class_name
        return self

    def set_input_read_only(self, input_read_only: bool):
        """
        设置输入框为只读。

        Args:
            input_read_only (bool): 是否只读。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.input_read_only = input_read_only
        return self

    def set_open(self, open: bool):
        """
        控制浮层显隐。

        Args:
            open (bool): 是否显示浮层。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.open = open
        return self

    def set_placeholder(self, placeholder: List[str]):
        """
        设置输入框占位文本。

        Args:
            placeholder (List[str]): 占位文本。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placeholder = placeholder
        return self

    def set_placement(self, placement: str):
        """
        设置浮层预设位置。

        Args:
            placement (str): 预设位置，如 'bottomLeft', 'bottomRight', 'topLeft', 'topRight'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.placement = placement
        return self

    def set_popup_style(self, popup_style: Any):
        """
        设置额外的弹出日历样式。

        Args:
            popup_style (Any): 弹出日历样式。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.popup_style = popup_style
        return self

    def set_size(self, size: str):
        """
        设置控件大小。

        Args:
            size (str): 控件大小，如 'large', 'middle', 'small'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_status(self, status: str):
        """
        设置校验状态。

        Args:
            status (str): 校验状态，如 'error', 'warning'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.status = status
        return self

    def set_suffix_icon(self, suffix_icon: Any):
        """
        设置自定义的选择框后缀图标。

        Args:
            suffix_icon (Any): 后缀图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.suffix_icon = suffix_icon
        return self

    def set_show_now(self, show_now: bool):
        """
        设置面板是否显示“此刻”按钮。

        Args:
            show_now (bool): 是否显示“此刻”按钮。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_now = show_now
        return self



from typing import Any, List, Optional, Union

from pydantic import Field, model_validator

from ..component import Component
from ..drawer.drawer import Drawer
from ..modal.modal import Modal


class Action(Component):
    component: Optional[str] = Field(default="action")
    label: Any = Field(default=None)
    block: bool = Field(default=False)
    batch: bool = Field(default=False)
    danger: bool = Field(default=False)
    disabled: bool = Field(default=False)
    ghost: bool = Field(default=False)
    icon: Optional[Any] = Field(default=None)
    shape: Optional[str] = Field(default=None)
    size: str = Field(default="default")
    type: str = Field(default="default")
    action_type: Optional[str] = Field(default=None, alias="actionType")
    submit_form: Any = Field(default=None, alias="submitForm")
    href: Optional[str] = Field(default=None)
    target: Optional[str] = Field(default=None)
    modal: Optional["Modal"] = Field(default=None)
    drawer: Optional["Drawer"] = Field(default=None)
    checked_children: Optional[Any] = Field(default=None, alias="checkedChildren")
    un_checked_children: Optional[Any] = Field(default=None, alias="unCheckedChildren")
    field_name: Optional[Any] = Field(default=None, alias="fieldName")
    field_value: Optional[Any] = Field(default=None, alias="fieldValue")
    confirm_title: str = Field(default="", alias="confirmTitle")
    confirm_text: str = Field(default="", alias="confirmText")
    confirm_type: str = Field(default="", alias="confirmType")
    api: Optional[str] = Field(default=None)
    reload: Optional[str] = Field(default=None)
    with_loading: bool = Field(default=False, alias="withLoading")

    @model_validator(mode="after")
    def init(self):
        self.set_key("action")
        return self

    def set_style(self, style: dict):
        self.style = style
        return self

    def set_label(self, label: Any):
        self.label = label
        return self

    def set_block(self, block: bool):
        self.block = block
        return self

    def set_batch(self, batch: bool):
        self.batch = batch
        return self

    def set_danger(self, danger: bool):
        self.danger = danger
        return self

    def set_disabled(self, disabled: bool):
        self.disabled = disabled
        return self

    def set_ghost(self, ghost: bool):
        self.ghost = ghost
        return self

    def set_icon(self, icon: Union[str, List[str]]):
        self.icon = icon
        return self

    def set_shape(self, shape: str):
        self.shape = shape
        return self

    def set_type(self, button_type: str, danger: bool):
        self.type = button_type
        self.danger = danger
        return self

    def set_size(self, size: str):
        self.size = size
        return self

    def set_action_type(self, action_type: str):
        self.action_type = action_type
        return self

    def set_submit_form(self, form_key: str):
        self.submit_form = form_key
        return self

    def set_href(self, href: str):
        self.href = href
        return self

    def set_target(self, target: str):
        self.target = target
        return self

    def set_link(self, href: str, target: str):
        self.set_href(href)
        self.set_target(target)
        self.action_type = "link"
        return self

    def set_modal(self, callback):
        component = Modal()
        self.modal = callback(component)
        return self

    def set_drawer(self, callback):
        component = Drawer()
        self.drawer = callback(component)
        return self

    def set_checked_children(self, checked_children: Any):
        self.checked_children = checked_children
        return self

    def set_un_checked_children(self, un_checked_children: Any):
        self.un_checked_children = un_checked_children
        return self

    def set_field_name(self, field_name: Any):
        self.field_name = field_name
        return self

    def set_field_value(self, field_value: Any):
        self.field_value = field_value
        return self

    def set_with_confirm(self, title: str, text: str, confirm_type: str):
        self.confirm_title = title
        self.confirm_text = text
        self.confirm_type = confirm_type
        return self

    def set_api(self, api: str):
        self.api = api
        self.action_type = "ajax"
        return self

    def set_reload(self, reload: str):
        self.reload = reload
        return self

    def set_with_loading(self, loading: bool):
        self.with_loading = loading
        return self



from typing import Any, List

from quark import Request, Resource, models
from quark.app import actions, searches
from quark.component.form import Rule, field


class Permission(Resource):
    """
    权限管理
    """

    async def init(self, request: Request):

        # 页面标题
        self.title = "权限"

        # 模型
        self.model = models.Permission

        return self

    async def fields(self, request: Request) -> List[Any]:
        """字段定义"""
        return [
            field.id("id", "ID"),
            field.text("name", "名称").set_rules(
                [
                    Rule.required("名称必须填写"),
                ]
            ),
            field.text("path", "路径").set_rules(
                [
                    Rule.required("路径必须填写"),
                ]
            ),
            field.select("method", "方法")
            .set_options(
                [
                    field.select_option("Any", "Any"),
                    field.select_option("GET", "GET"),
                    field.select_option("HEAD", "HEAD"),
                    field.select_option("OPTIONS", "OPTIONS"),
                    field.select_option("POST", "POST"),
                    field.select_option("PUT", "PUT"),
                    field.select_option("PATCH", "PATCH"),
                    field.select_option("DELETE", "DELETE"),
                ]
            )
            .set_filters(True)
            .set_default_value("GET"),
            field.textarea("remark", "备注"),
        ]

    async def searches(self, request: Request) -> List[Any]:
        """搜索项定义"""
        return [
            searches.Input("name", "名称"),
            searches.Input("path", "路径"),
        ]

    async def actions(self, request: Request) -> List[Any]:
        """行为定义"""
        return [
            actions.CreateModal(self),
            actions.BatchDelete(),
            actions.SyncPermission(),
            actions.EditModal(self),
            actions.Delete(),
            actions.FormSubmit(),
            actions.FormReset(),
            actions.FormBack(),
            actions.FormExtraBack(),
        ]



from .batch_delete_role import BatchDeleteRole
from .batch_delete import BatchDelete
from .batch_disable import BatchDisable
from .batch_enable import BatchEnable
from .batch_export import BatchExport
from .change_status import ChangeStatus
from .create_drawer import CreateDrawer
from .create_link import CreateLink
from .create_modal import CreateModal
from .data_scope import DataScope
from .delete_role import DeleteRole
from .delete_special import DeleteSpecial
from .delete import Delete
from .detail_drawer import DetailDrawer
from .detail_link import DetailLink
from .detail_modal import DetailModal
from .edit_drawer import EditDrawer
from .edit_link import EditLink
from .edit_modal import EditModal
from .export import Export
from .form_back import FormBack
from .form_extra_back import FormExtraBack
from .form_reset import FormReset
from .form_step import FormStep
from .form_submit import FormSubmit
from .import_action import Import
from .menu_create_drawer import MenuCreateDrawer
from .menu_edit_drawer import MenuEditDrawer
from .more import More
from .status import Status
from .sync_permission import SyncPermission



from datetime import datetime
from io import BytesIO

from openpyxl import Workbook
from openpyxl.utils import get_column_letter

from quark import Request, Response


class ImportTemplateRequest:

    # 请求对象
    request: Request = None

    # 列表页字段
    fields: list = None

    def __init__(
        self,
        request: Request,
        fields: list,
    ):
        self.request = request
        self.fields = fields

    def handle(self):
        """
        处理导出导入模板逻辑
        """
        export_titles = [
            self.get_field_label_with_remark(field) for field in self.fields
        ]

        wb = Workbook()
        ws = wb.active
        ws.title = "Sheet1"

        # 写入表头
        for col_idx, title in enumerate(export_titles, start=1):
            ws[f"{get_column_letter(col_idx)}1"] = title

        # 返回 Excel 文件流
        output = BytesIO()
        wb.save(output)
        output.seek(0)

        filename = f"data_{datetime.now().strftime('%Y%m%d%H%M%S')}.xlsx"
        headers = {
            "Content-Disposition": f"attachment; filename={filename}",
            "Content-Type": "application/octet-stream",
        }

        return Response(
            output.getvalue(),
            mimetype="application/vnd.openxmlformats-officedocument.spreadsheetml.sheet",
            headers=headers,
        )

    def get_field_label_with_remark(self, field):
        """
        获取字段 label + 提示信息
        """
        label = field.get("label", "")
        remark = self.get_field_remark(field)
        if remark:
            return f"{label}（{remark}）"
        return label

    def get_field_remark(self, field):
        """
        根据字段组件类型和规则生成备注信息
        """
        component = field.get("component")
        rules = field.get("rules", [])
        mode = field.get("mode")
        options = field.get("options", "")

        remarks = []

        if component == "inputNumberField":
            remarks.append("数字格式")
        elif component == "textField":
            pass  # 无特殊说明
        elif component == "selectField":
            if mode == "multiple":
                remarks.append(f"可多选：{options}；多值请用“,”分割")
            else:
                remarks.append(f"可选项：{options}")
        elif component == "cascaderField":
            remarks.append("级联格式，例如：省，市，县")
        elif component == "checkboxField":
            remarks.append(f"可多选项：{options}；多值请用“,”分割")
        elif component == "radioField":
            remarks.append(f"可选项：{options}")
        elif component == "switchField":
            remarks.append(f"可选项：{options}")
        elif component == "dateField":
            remarks.append("日期格式，例如：1987-02-15")
        elif component == "datetimeField":
            remarks.append("日期时间格式，例如：1987-02-15 20:00:00")

        rule_messages = self.get_rule_messages(rules)
        if rule_messages:
            remarks.append(f"条件：{rule_messages}")

        return "，".join(remarks)

    def get_rule_messages(self, rules):
        """
        解析规则并返回中文描述
        """
        messages = []
        for rule in rules:
            if isinstance(rule, dict):
                rule_type = rule.get("type")
                value = rule.get("value")
                if rule_type == "required":
                    messages.append("必填")
                elif rule_type == "min":
                    messages.append(f"大于{value}个字符")
                elif rule_type == "max":
                    messages.append(f"小于{value}个字符")
                elif rule_type == "email":
                    messages.append("必须为邮箱格式")
                elif rule_type == "numeric":
                    messages.append("必须为数字格式")
                elif rule_type == "url":
                    messages.append("必须为链接格式")
                elif rule_type == "integer":
                    messages.append("必须为整数格式")
                elif rule_type == "date":
                    messages.append("必须为日期格式")
                elif rule_type == "boolean":
                    messages.append("必须为布尔格式")
                elif rule_type == "unique":
                    messages.append("不可重复")
        return "，".join(messages)



from typing import Any

from tortoise.queryset import QuerySet

from quark import Message, Request

from ..performs_queries import PerformsQueries


class EditableRequest:

    # 请求对象
    request: Request = None

    # 资源对象
    resource: Any = None

    # 查询对象
    query: QuerySet = None

    def __init__(
        self,
        request: Request,
        resource: Any,
        query: QuerySet,
    ):
        self.request = request
        self.resource = resource
        self.query = query

    async def handle(self):
        data = self.request.query_params

        id = data.get("id")
        if not id:
            return Message.error("id不能为空")

        model_instance = await self.query.get(id=id)
        if not model_instance:
            return Message.error("记录不存在")

        field = None
        value = None
        for k, v in data.items():
            if v == "true":
                v = 1
            elif v == "false":
                v = 0

            if k not in ["id", "_t"]:
                field = k
                value = v

        if not field or value is None:
            return Message.error("参数错误")

        try:
            await self.resource.before_editable(self.request, id, field, value)
        except Exception as e:
            return Message.error(str(e))

        # 构建查询并更新数据
        try:
            await PerformsQueries(self.request).build_editable_query(self.query).update(
                **{field: value}
            )
        except Exception as e:
            return Message.error(str(e))

        # 行内编辑执行后回调
        try:
            await self.resource.after_editable(self.request, id, field, value)
        except Exception as e:
            return Message.error(str(e))

        return Message.success("操作成功")



from typing import Any, Dict, List, Optional

from .base import Base


class DataSource(Base):
    """
    表示 Transfer 组件数据源的类。

    Attributes:
        key (Any): 数据源项的主键。
        title (str): 数据源项的标题。
        description (str): 数据源项的描述信息。
        disabled (bool): 数据源项是否禁用，默认为 False。
    """

    key: Any = None
    title: Optional[str] = None
    description: Optional[str] = None
    disabled: bool = False


class Transfer(Base):

    component: str = "transferField"
    """
    组件名称
    """

    data_source: List[DataSource] = []
    """
    数据源，其中的数据将会被渲染到左边一栏中，target_keys 中指定的除外
    """

    disabled: bool = False
    """
    是否禁用，默认值为 False
    """

    selections_icon: Optional[Any] = None
    """
    自定义下拉菜单图标
    """

    filter_option: Optional[Any] = None
    """
    根据搜索内容进行筛选，接收 inputValue option 两个参数，
            当 option 符合筛选条件时，应返回 true，反之则返回 false
    """

    footer: Optional[Any] = None
    """
    底部渲染函数
    """

    list_style: Optional[Dict[str, Any]] = None
    """
    两个穿梭框的自定义样式
    """

    locale: Optional[Dict[str, str]] = None
    """
    各种语言,{ itemUnit: 项, itemsUnit: 项, searchPlaceholder: 请输入搜索内容 }
    """

    one_way: bool = False
    """
    展示为单向样式，默认值为 False
    """

    operations: List[str] = []
    """
    操作文案集合，顺序从上至下
    """

    operation_style: Optional[Dict[str, Any]] = None
    """
    操作栏的自定义样式
    """

    pagination: Optional[Any] = None
    """
    使用分页样式，自定义渲染列表下无效
    """

    select_all_labels: Optional[Any] = None
    """
    自定义顶部多选框标题的集合
    """

    selected_keys: List[str] = []
    """
    设置哪些项应该被选中
    """

    show_search: bool = False
    """
    是否显示搜索框，默认值为 False
    """

    show_select_all: bool = False
    """
    是否展示全选勾选框，默认值为 False
    """

    status: Optional[str] = None
    """
    设置校验状态,'error' | 'warning'
    """

    target_keys: List[str] = []
    """
    显示在右侧框数据的 key 集合
    """

    titles: List[str] = []
    """
    标题集合，顺序从左至右
    """

    style: Optional[Dict[str, Any]] = None
    """
    自定义样式
    """

    default_value: Optional[Any] = None
    """
    默认的选中项
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    def build_data_source(
        self, items: Any, key_name: str, title_name: str, description_name: str
    ) -> List[DataSource]:
        """
        使用反射构建数据源。

        Args:
            items (Any): 包含数据源的对象。
            key_name (str): 主键字段名。
            title_name (str): 标题字段名。
            description_name (str): 描述字段名。

        Returns:
            List[DataSource]: 构建好的数据源列表。
        """
        options: List[DataSource] = []

        if not isinstance(items, list):
            return options

        for item in items:
            if isinstance(item, dict):
                key = item.get(key_name)
                title = item.get(title_name, "")
                description = item.get(description_name, "")
            else:
                key = getattr(item, key_name, None)
                title = getattr(item, title_name, "")
                description = getattr(item, description_name, "")

            if key is not None and title:
                option = DataSource(key=key, title=title, description=description)
                options.append(option)

        return options

    def list_to_data_source(
        self, list_: Any, key_name: str, title_name: str, description_name: str
    ) -> List[DataSource]:
        """
        将列表转换为数据源。

        Args:
            list_ (Any): 包含数据的列表。
            key_name (str): 主键字段名。
            title_name (str): 标题字段名。
            description_name (str): 描述字段名。

        Returns:
            List[DataSource]: 转换后的数据源列表。
        """
        return self.build_data_source(list_, key_name, title_name, description_name)

    def set_data_source(self, *data_source: Any):
        """
        设置数据源。

        Args:
            *data_source: 可变参数，不同数量有不同含义。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        if len(data_source) == 1:
            if isinstance(data_source[0], list) and all(
                isinstance(item, DataSource) for item in data_source[0]
            ):
                self.data_source = data_source[0]
        elif len(data_source) == 4:
            self.data_source = self.list_to_data_source(
                data_source[0], data_source[1], data_source[2], data_source[3]
            )
        return self

    def set_selections_icon(self, selections_icon: Any):
        """
        设置自定义下拉菜单图标。

        Args:
            selections_icon (Any): 下拉菜单图标。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.selections_icon = selections_icon
        return self

    def set_filter_option(self, filter_option: Any):
        """
        设置根据搜索内容进行筛选的函数。

        Args:
            filter_option (Any): 筛选函数。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.filter_option = filter_option
        return self

    def set_footer(self, footer: Any):
        """
        设置底部渲染函数。

        Args:
            footer (Any): 底部渲染函数。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.footer = footer
        return self

    def set_list_style(self, list_style: Dict[str, Any]):
        """
        设置两个穿梭框的自定义样式。

        Args:
            list_style (Dict[str, Any]): 样式字典。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.list_style = list_style
        return self

    def set_locale(self, locale: Dict[str, str]):
        """
        设置各种语言配置。

        Args:
            locale (Dict[str, str]): 语言配置字典。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.locale = locale
        return self

    def set_one_way(self, one_way: bool):
        """
        设置展示为单向样式。

        Args:
            one_way (bool): 是否展示为单向样式。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.one_way = one_way
        return self

    def set_operations(self, operations: List[str]):
        """
        设置操作文案集合。

        Args:
            operations (List[str]): 操作文案列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.operations = operations
        return self

    def set_operation_style(self, operation_style: Dict[str, Any]):
        """
        设置操作栏的自定义样式。

        Args:
            operation_style (Dict[str, Any]): 样式字典。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.operation_style = operation_style
        return self

    def set_pagination(self, pagination: Any):
        """
        设置使用分页样式。

        Args:
            pagination (Any): 分页样式配置。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.pagination = pagination
        return self

    def set_select_all_labels(self, select_all_labels: Any):
        """
        设置自定义顶部多选框标题的集合。

        Args:
            select_all_labels (Any): 多选框标题集合。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.select_all_labels = select_all_labels
        return self

    def set_selected_keys(self, selected_keys: List[str]):
        """
        设置哪些项应该被选中。

        Args:
            selected_keys (List[str]): 选中项的 key 列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.selected_keys = selected_keys
        return self

    def set_show_search(self, show_search: bool):
        """
        设置是否显示搜索框。

        Args:
            show_search (bool): 是否显示搜索框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_search = show_search
        return self

    def set_show_select_all(self, show_select_all: bool):
        """
        设置是否展示全选勾选框。

        Args:
            show_select_all (bool): 是否展示全选勾选框。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.show_select_all = show_select_all
        return self

    def set_status(self, status: str):
        """
        设置校验状态。

        Args:
            status (str): 校验状态，如 'error', 'warning'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.status = status
        return self

    def set_target_keys(self, target_keys: List[str]):
        """
        设置显示在右侧框数据的 key 集合。

        Args:
            target_keys (List[str]): key 集合。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.target_keys = target_keys
        return self

    def set_titles(self, titles: List[str]):
        """
        设置标题集合。

        Args:
            titles (List[str]): 标题列表。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.titles = titles
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from typing import Dict, List, Optional, Any
from .base import Base


class Option(Base):
    """
    表示搜索组件选项的类。

    Attributes:
        label (str): 选项显示的标签。
        value (Any): 选项对应的值。
        disabled (bool): 选项是否禁用，默认为 False。
    """

    label: str
    value: Any
    disabled: bool = False


class Search(Base):

    component: str = "searchField"
    """
    组件名称
    """

    allow_clear: bool = True
    """
    可以点击清除图标删除内容，默认值为 True
    """

    placeholder: str = "请输入要搜索的内容"
    """
    占位符，默认值为 "请输入要搜索的内容"
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项
    """

    disabled: bool = False
    """
    整组失效，默认值为 False
    """

    options: List[Option] = []
    """
    可选项数据源
    """

    option_type: Optional[str] = None
    """
    用于设置 Search options 类型 default | button
    """

    size: Optional[str] = None
    """
    大小，只对按钮样式生效, large | middle | small
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    style: Dict[str, Any] = {}
    """
    自定义样式，默认值为空字典
    """

    def build_options(
        self, items: Any, label_name: str, value_name: str
    ) -> List[Option]:
        """
        使用反射构建树结构。

        Args:
            items (Any): 包含选项数据的对象。
            label_name (str): 标签字段名。
            value_name (str): 值字段名。

        Returns:
            List[Option]: 构建好的选项列表。
        """
        options: List[Option] = []

        if not isinstance(items, list):
            return options

        for item in items:
            if hasattr(item, label_name) and hasattr(item, value_name):
                label = getattr(item, label_name)
                value = getattr(item, value_name)
                option = Option(label=label, value=value)
                options.append(option)

        return options

    def list_to_options(
        self, list_data: Any, label_name: str, value_name: str
    ) -> List[Option]:
        """
        将列表数据转换为选项列表。

        Args:
            list_data (Any): 列表数据。
            label_name (str): 标签字段名。
            value_name (str): 值字段名。

        Returns:
            List[Option]: 转换后的选项列表。
        """
        return self.build_options(list_data, label_name, value_name)

    def set_options(self, *options: Any):
        """
        设置属性，示例：[Option(value=1, label="男"), Option(value=2, label="女")]
        或者 set_options(options, "label_name", "value_name")

        Args:
            *options: 可变参数，不同数量有不同含义。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        if len(options) == 1:
            if isinstance(options[0], list) and all(
                isinstance(option, Option) for option in options[0]
            ):
                self.options = options[0]
                return self
        if len(options) == 3:
            self.options = self.list_to_options(options[0], options[1], options[2])
        return self

    def set_api(self, api: str):
        """
        设置获取数据接口。

        Args:
            api (str): 接口地址。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.api = api
        return self

    def set_option_type(self, option_type: str):
        """
        设置 Search options 类型。

        Args:
            option_type (str): 类型，如 'default', 'button'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.option_type = option_type
        return self

    def set_size(self, size: str):
        """
        设置控件大小。

        Args:
            size (str): 大小，如 'large', 'middle', 'small'。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.size = size
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from .search import Search


class TreeSelect(Search):

    def __init__(self, column: str = "", name: str = ""):
        self.component = "treeSelectField"
        return self

    # 设置 Option
    def option(self, title: str, value):
        return {"value": value, "title": title}

    # 可选项数据源
    #
    # 示例：
    # set_tree_data([{"value": "zhejiang", "title": "Zhejiang"}])
    #
    # 或者
    #
    # set_tree_data(options, parent_key_name, title_name, value_name)
    #
    # 或者
    #
    # set_tree_data(options, 0, parent_key_name, title_name, value_name)
    def set_tree_data(self, *args):
        if len(args) == 1 and isinstance(args[0], list):
            self.tree_select_options = args[0]
            return self

        if len(args) == 4:
            self.tree_select_options = self.list_to_tree_data(
                args[0], 0, args[1], args[2], args[3]
            )
            return self

        if len(args) == 5:
            self.tree_select_options = self.list_to_tree_data(
                args[0], args[1], args[2], args[3], args[4]
            )
            return self

        return self

    @staticmethod
    def list_to_tree_data(data, parent_id, parent_key, title_key, value_key):
        """模拟从列表结构生成树形数据"""
        tree_data = []
        data_map = {}

        # 构建所有节点的映射关系
        for item in data:
            item_id = item[value_key]
            data_map[item_id] = {
                "title": item[title_key],
                "value": item_id,
                "children": [],
            }

        # 根据 parentId 构建父子关系
        for item in data:
            node = data_map[item[value_key]]
            if item[parent_key] == parent_id:
                tree_data.append(node)
            else:
                parent_node = data_map.get(item[parent_key], None)
                if parent_node is not None:
                    parent_node["children"].append(node)

        return tree_data



from quark import Request

from .action import Action


class Link(Action):
    """
    表示一个链接行为组件，用于页面跳转。
    """

    # 行为类型
    action_type: str = "link"

    # 获取跳转链接，默认为空字符串
    href: str = ""

    # 相当于 a 链接的 target 属性，href 存在时生效，默认为 _self
    target: str = "_self"

    async def get_href(self, request: Request) -> str:
        """
        获取跳转链接地址
        """
        return self.href

    async def get_target(self, request: Request) -> str:
        """
        获取链接打开方式，相当于 a 标签的 target 属性
        """
        return self.target



from typing import Dict, Any, Optional
from .base import Base


class Map(Base):

    component: str = "mapField"
    """
    组件名称
    """

    default_value: Optional[Any] = None
    """
    默认选中的选项
    """

    disabled: bool = False
    """
    整组失效，默认 False
    """

    style: Dict[str, Any] = {"height": 500, "width": "100%", "marginTop": "10px"}
    """
    自定义样式，默认包含高度、宽度和上边距
    """

    value: Optional[Any] = None
    """
    指定选中项
    """

    zoom: int = 14
    """
    缩放级别，默认 14
    """

    map_key: str = "70ac74a1443326e66e51a4255700a4e2"
    """
    地图 Key
    """

    map_security_js_code: str = "5c4fc57d6cba5efd1c15c988d18d2a78"
    """
    地图安全密钥
    """

    def set_zoom(self, zoom: int):
        """
        设置缩放级别。

        Args:
            zoom (int): 缩放级别值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.zoom = zoom
        return self

    def set_map_key(self, key: str):
        """
        设置高德地图 key。

        Args:
            key (str): 地图 key 值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.map_key = key
        return self

    def set_map_security_js_code(self, security_js_code: str):
        """
        设置高德地图安全秘钥。

        Args:
            security_js_code (str): 安全秘钥值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.map_security_js_code = security_js_code
        return self

    def set_width(self, width: Any):
        """
        设置地图宽度。

        Args:
            width (Any): 地图宽度值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        style = self.style.copy()
        style["width"] = width
        self.style = style
        return self

    def set_height(self, height: Any):
        """
        设置地图高度。

        Args:
            height (Any): 地图高度值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        style = self.style.copy()
        style["height"] = height
        self.style = style
        return self

    def set_position(self, longitude: str, latitude: str):
        """
        设置坐标位置。

        Args:
            longitude (str): 经度值。
            latitude (str): 纬度值。

        Returns:
            Component: 返回当前实例，支持链式调用。
        """
        self.value = {"longitude": longitude, "latitude": latitude}
        return self

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self



from typing import Any, List

from quark.template.action import Dropdown


class More(Dropdown):
    def __init__(self, name: str = "更多"):
        self.name = name
        self.arrow = True
        self.placement = "bottomLeft"
        self.trigger = ["hover"]
        self.overlay_style = {"zIndex": 999}
        self.type = "link"
        self.size = "small"
        self.set_only_on_index_table_row(True)

    def set_actions(self, actions: List[Any]) -> Any:
        self.actions = actions
        return self



import platform
import sys

import psutil

from quark import config
from quark.component.descriptions.fields.text import Text
from quark.template.metric.descriptions import Descriptions


class SystemInfo(Descriptions):
    title: str = "系统信息"
    col: int = 12

    async def calculate(self) -> any:
        memory = psutil.virtual_memory()
        cpu_percent = psutil.cpu_percent(interval=1)
        return self.result(
            [
                Text().set_label("应用名称").set_value(config.get("APP_NAME")),
                Text().set_label("应用版本").set_value(config.get("APP_VERSION")),
                Text().set_label("Python版本").set_value(sys.version.split(" ")[0]),
                Text()
                .set_label("服务器操作系统")
                .set_value(f"{platform.system()} {platform.machine()}"),
                Text()
                .set_label("内存信息")
                .set_value(f"{memory.total // (1024 * 1024)}MB / {memory.percent}%"),
                Text().set_label("CPU使用率").set_value(f"{cpu_percent}%"),
            ]
        )



import json
from datetime import datetime
from typing import Any, Dict, Optional

from tortoise.queryset import QuerySet

from quark import Message, Request

from ..performs_queries import PerformsQueries


class EditRequest:
    """
    用于处理资源编辑页数据填充和初始化数据获取。
    """

    # 请求对象
    request: Request = None

    # 资源对象
    resource: Any = None

    # 查询对象
    query: QuerySet = None

    # 字段
    fields: Optional[Any] = None

    def __init__(
        self,
        request: Request,
        resource: Any,
        query: QuerySet,
        fields: Optional[Any],
    ):
        self.request = request
        self.resource = resource
        self.query = query
        self.fields = fields

    async def fill_data(self) -> Dict[str, Any]:
        """
        获取并解析编辑表单数据。
        """
        id_value = self.request.query_params.get("id", "")
        if not id_value:
            return {}

        # 构建查询
        result = (
            await PerformsQueries(self.request).build_edit_query(self.query).first()
        )
        if not result:
            return {}

        # 将查询结果转为 dict
        if hasattr(result, "to_dict"):
            result_dict = result.to_dict()
        elif hasattr(result, "__dict__"):
            result_dict = dict(result.__dict__)
        else:
            result_dict = dict(result)

        # 填充字段数据
        fields_data = {}
        for field in self.fields:
            name = getattr(field, "name", None)
            if not name or name not in result_dict:
                continue

            value = result_dict[name]
            component = getattr(field, "component", "")
            field_value: Any = value

            # 时间字段格式化
            if component in ["datetimeField", "dateField"]:
                format_str = getattr(field, "format", "%Y-%m-%d %H:%M:%S")
                format_str = (
                    format_str.replace("YYYY", "%Y")
                    .replace("MM", "%m")
                    .replace("DD", "%d")
                    .replace("HH", "%H")
                    .replace("mm", "%M")
                    .replace("ss", "%S")
                )
                if isinstance(value, datetime):
                    field_value = value.strftime(format_str)

            fields_data[name] = field_value

        return fields_data

    async def values(self) -> Any:
        """
        获取表单初始化数据，并调用显示前回调。
        """
        data = await self.fill_data()

        # 编辑前回调
        data = await self.resource.before_editing(self.request, data)

        # 解析嵌套 JSON 字符串为对象
        parsed_data = {}
        for key, value in data.items():
            if isinstance(value, str):
                try:
                    parsed = json.loads(value)
                    if isinstance(parsed, (dict, list)):
                        value = parsed
                except json.JSONDecodeError:
                    pass
            parsed_data[key] = value

        return Message.success("获取成功", parsed_data)



from typing import Any, Optional

from quark import Request
from quark.template.action import Link


class DetailLink(Link):

    def __init__(self, name: Optional[str] = None):
        self.name = name or "详情"
        self.type = "link"
        self.size = "small"
        self.set_only_on_index_table_row(True)

    def init(self, request: Request) -> Any:
        # 初始化按钮属性（如果需要额外初始化逻辑，可以写这里）
        return self

    async def get_href(self, request: Request) -> str:
        # 替换路径中的 "/index" 为 "/detail&id=${id}"
        href = request.url.path.replace("/index", "/detail&id=${id}")
        return f"#/layout/index?api={href}"



from quark.component.statistic.statistic import Statistic
from quark.template.metric.value import Value
from quark.services.attachment import AttachmentService


class TotalFile(Value):
    title: str = "文件数量"
    col: int = 6

    async def calculate(self) -> Statistic:
        """计算数值"""
        count = await AttachmentService().count_by_type("FILE")
        return self.count(count).set_value_style({"color": "#cf1322"})



import json
from datetime import datetime
from typing import Any, Dict, Optional

from tortoise.queryset import QuerySet

from quark import Message, Request

from ..performs_queries import PerformsQueries
from ..resolves_actions import ResolvesActions


class DetailRequest:
    """
    用于处理资源详情页数据填充和初始化数据获取。
    """

    # 请求对象
    request: Request = None

    # 资源对象
    resource: Any = None

    # 查询对象
    query: QuerySet = None

    # 字段
    fields: Optional[Any] = None

    def __init__(
        self,
        request: Request,
        resource: Any,
        query: QuerySet,
        fields: Optional[Any],
    ):
        self.request = request
        self.resource = resource
        self.query = query
        self.fields = fields

    async def fill_data(self) -> Dict[str, Any]:
        """
        获取并解析详情页数据
        """
        result = {}
        id_value = self.request.query_params.get("id", "")
        if not id_value:
            return result

        # 构建查询
        result = (
            await PerformsQueries(self.request).build_detail_query(self.query).first()
        )
        fields = {}
        for field in self.fields:
            component = getattr(field, "component", None)
            name = getattr(field, "name", "")

            if component == "actionField":
                # 行为字段
                items = getattr(field, "items", [])
                callback = field.callback

                if callback:
                    items = await callback(result)

                parsed_items = []
                for action in items:
                    parsed_items.append(
                        await ResolvesActions(self.request).build_action(action)
                    )

                fields[name] = parsed_items

            else:
                # 普通字段
                callback = field.callback
                if callback:
                    fields[name] = await callback(result)
                else:
                    value = getattr(result, name, None)
                    if value is None:
                        continue

                    if isinstance(value, str):
                        # JSON 解析
                        try:
                            value = json.loads(value)
                        except json.JSONDecodeError:
                            pass

                    # 时间字段处理
                    if component in ("datetimeField", "dateField"):
                        format_str = getattr(field, "format", "%Y-%m-%d %H:%M:%S")
                        format_str = (
                            format_str.replace("YYYY", "%Y")
                            .replace("MM", "%m")
                            .replace("DD", "%d")
                            .replace("HH", "%H")
                            .replace("mm", "%M")
                            .replace("ss", "%S")
                        )
                        if isinstance(value, datetime):
                            value = value.strftime(format_str)

                    fields[name] = value

        return fields

    async def values(self) -> Any:
        """
        获取表单初始化数据，并调用显示前回调。
        """
        data = await self.fill_data()

        # 显示前回调
        data = await self.resource.before_detail_showing(self.request, data)

        return Message.success("获取成功", data)



from typing import Any, Dict, List

from pydantic import BaseModel, Field

from quark import Message, Request
from quark.schemas import FileInfo, MinioConfig, OSSConfig
from quark.storage import Storage


class Upload(BaseModel):
    """
    文件上传模板类

    处理文件上传的核心逻辑，支持普通文件上传和Base64格式上传
    """

    # 上传限制
    limit_size: int = Field(default=0)

    # 允许的文件类型
    limit_type: List[str] = Field(default_factory=list)

    # 限制图片宽度
    limit_image_width: int = Field(default=0)

    # 限制图片高度
    limit_image_height: int = Field(default=0)

    # 存储驱动
    driver: str = Field(default="local")

    # 保存路径
    save_path: str = Field(default="uploads")

    # OSS配置
    oss_config: OSSConfig = Field(default=None)

    # Minio配置
    minio_config: MinioConfig = Field(default=None)

    async def init(self, request: Request):
        """初始化"""
        return self

    async def handle(self, request: Request) -> Dict[str, Any]:
        """
        执行文件上传
        """
        # 解析 multipart/form-data
        form = await request.form()

        # 获取字段名为 file 的文件
        file = form.get("file")

        try:
            limit_w = request.query_params.get("limitW", "")
            limit_h = request.query_params.get("limitH", "")

            limit_image_width = self.limit_image_width
            if limit_w != "":
                try:
                    limit_image_width = int(limit_w)
                except ValueError:
                    pass  # 保持原值

            limit_image_height = self.limit_image_height
            if limit_h != "":
                try:
                    limit_image_height = int(limit_h)
                except ValueError:
                    pass  # 保持原值

            storage = Storage(
                file=file,
                driver=self.driver,
                oss_config=self.oss_config,
                minio_config=self.minio_config,
                limit_image_height=limit_image_height,
                limit_image_width=limit_image_width,
                limit_size=self.limit_size,
                limit_type=self.limit_type,
                with_image_extra=True,
                rand_name=True,
            )

            # 上传前回调
            before_handle_result = await self.before_handle(request, storage)

            if before_handle_result is not None:
                return before_handle_result

            result = await storage.path(self.save_path).save()

            return await self.after_handle(request, result)
        except Exception as e:
            return Message.error(str(e))

    async def handle_from_base64(self, request: Request) -> Dict[str, Any]:
        """
        通过Base64执行上传
        """

        try:
            limit_w = request.query_params.get("limitW", "")
            limit_h = request.query_params.get("limitH", "")

            try:
                data = await request.json()
            except ValueError as e:
                return Message.error(str(e))

            if "file" not in data:
                return Message.error("参数错误")

            limit_image_width = self.limit_image_width
            if limit_w != "":
                try:
                    limit_image_width = int(limit_w)
                except ValueError:
                    pass

            limit_image_height = self.limit_image_height
            if limit_h != "":
                try:
                    limit_image_height = int(limit_h)
                except ValueError:
                    pass

            storage = Storage(
                file_base64_str=data["file"],
                driver=self.driver,
                oss_config=self.oss_config,
                minio_config=self.minio_config,
                limit_image_height=limit_image_height,
                limit_image_width=limit_image_width,
                limit_size=self.limit_size,
                limit_type=self.limit_type,
                with_image_extra=True,
                rand_name=True,
            )

            # 上传前回调
            before_handle_result = await self.before_handle(request, storage)

            if before_handle_result is not None:
                return before_handle_result

            result = await storage.path(self.save_path).save()

            return await self.after_handle(request, result)

        except Exception as e:
            return Message.error(str(e))

    async def before_handle(self, request: Request, storage: Storage) -> Any:
        """
        上传前回调
        """
        return None

    async def after_handle(self, request: Request, result: FileInfo) -> Any:
        """
        上传后回调
        """
        return Message.success("上传成功", result)



from typing import Any, Dict, List

from tortoise import Model

from quark import Message, Request

from ..component.card.card import Card
from ..component.form.form import Form
from ..component.tabs.tabs import Tabs
from ..utils import is_creating, is_editing
from .request.store import StoreRequest
from .resolves_fields import ResolvesFields


class ResourceForm:
    """
    表单相关的方法
    """

    async def form_api(self, request: Request) -> str:
        """
        获取表单提交 API 地址
        """
        return ""

    async def form_title(self, request: Request) -> str:
        """
        获取表单标题
        """
        title = self.title
        if is_creating(request):
            return f"创建{title}"
        elif is_editing(request):
            return f"编辑{title}"
        else:
            return title

    async def form_component_render(
        self,
        request: Request,
        title: str,
        extra: Any,
        api: str,
        fields: Any,
        actions: List[Any],
        data: Dict[str, Any],
    ) -> Any:
        """
        渲染表单组件，支持 Tab 或 Card 两种形式
        """
        if isinstance(fields, list) and len(fields) > 0:
            first_item = fields[0]
            component_type = getattr(first_item, "component", "")
            if component_type == "tabPane":
                return await self.form_within_tabs(
                    request, title, extra, api, fields, actions, data
                )
        return await self.form_within_card(
            request, title, extra, api, fields, actions, data
        )

    async def form_within_card(
        self,
        request: Request,
        title: str,
        extra: Any,
        api: str,
        fields: Any,
        actions: List[Any],
        data: Dict[str, Any],
    ) -> Card:
        """
        在卡片中渲染表单
        """
        form = (
            self.form.set_style({"padding": "24px"})
            .set_api(api)
            .set_actions(actions)
            .set_body(fields)
            .set_initial_values(data)
        )

        return (
            Card()
            .set_title(title)
            .set_header_bordered(True)
            .set_extra(extra)
            .set_body(form)
        )

    async def form_within_tabs(
        self,
        request: Request,
        title: str,
        extra: Any,
        api: str,
        fields: Any,
        actions: List[Any],
        data: Dict[str, Any],
    ) -> Form:
        """
        在标签页中渲染表单
        """
        tabs_component = Tabs().set_tab_panes(fields).set_tab_bar_extra_content(extra)

        return (
            self.form.set_style({"backgroundColor": "#fff", "paddingBottom": "20px"})
            .set_api(api)
            .set_actions(actions)
            .set_body(tabs_component)
            .set_initial_values(data)
        )

    async def before_form_showing(self, request: Request) -> Any:
        """
        表单显示前回调
        """
        return {}

    async def form_handle(
        self, request: Request, model: Model, data: Dict[str, Any]
    ) -> Any:
        """
        表单提交处理
        """

        fields = ResolvesFields(
            request, await self.fields(request)
        ).creation_fields_without_when()

        return await StoreRequest(
            request=request, resource=self, model=model, fields=fields
        ).handle(data)

    async def before_saving(
        self, request: Request, submit_data: Dict[str, Any]
    ) -> Dict[str, Any]:
        """
        保存数据前回调
        """
        return submit_data

    async def after_imported(
        self, request: Request, id_: int, data: Dict[str, Any], result: Any
    ):
        """
        导入数据后回调
        """

    async def after_saved(
        self, request: Request, id: int, data: Dict[str, Any], result: Any
    ):
        """
        保存数据后回调
        """

    async def after_saved_redirect_to(
        self, request: Request, id: int, data: Dict[str, Any]
    ) -> Any:
        """
        保存数据后跳转处理
        """
        redirect_url = "/layout/index?api=" + "/api/admin/{resource}/index"
        redirect_url = redirect_url.replace(
            "{resource}", request.path_params.get("resource", "")
        )

        return Message.redirect_to("操作成功", redirect_url)



from typing import List

from tortoise.queryset import QuerySet

from quark import Message, Request
from quark.template.action import Action


class ChangeStatus(Action):

    def __init__(self, name: str = "<%= (status==1 ? '禁用' : '启用') %>"):
        self.name = name
        self.type = "link"
        self.size = "small"
        self.reload = "table"
        self.set_only_on_index_table_row(True)
        self.with_confirm(
            "确定要<%= (status==1 ? '禁用' : '启用') %>数据吗？", "", "pop"
        )

    def get_api_params(self) -> List[str]:
        return ["id", "status"]

    async def handle(self, request: Request, query: QuerySet):
        status = request.query_params.get("status")
        if status is None:
            return Message.error("参数错误")

        try:
            # 切换状态逻辑
            field_status = 0 if status == "1" else 1

            id_param = request.query_params.get("id")
            if not id_param:
                return Message.error("缺少 ID 参数")

            ids = [int(i) for i in id_param.split(",")]
            await query.filter(id__in=ids).update(status=field_status)
            return Message.success("操作成功")
        except Exception as e:
            return Message.error(str(e))



from quark.template.action import Action


class FormBack(Action):

    def __init__(self):

        # 文字
        self.name = "返回上一页"

        # 类型
        self.type = "default"

        # 行为类型
        self.action_type = "back"

        # 在表单页展示
        self.set_show_on_form()

        # 在详情页展示
        self.set_show_on_detail()



from typing import Any
from fastapi import Request
from tortoise.models import Model
from quark.template.search import DatetimeRange


class DatetimeRange(DatetimeRange):
    """日期范围组件"""

    def __init__(self, column: str = "", name: str = ""):
        self.column = column
        self.name = name

    def apply(self, request: Request, query: Model, value: Any) -> Model:
        filters = {
            f"{self.column}__gte": value[0],
            f"{self.column}__lte": value[1],
        }
        return query.filter(**filters)



from typing import Any, Dict, List
from .base import Base


class DateRange(Base):

    component: str = "dateRangeField"
    """
    组件名称
    """

    allow_clear: bool = True
    """
    是否支持清除操作
    """

    auto_focus: bool = False
    """
    是否自动获取焦点
    """

    bordered: bool = True
    """
    是否显示边框
    """

    class_name: str = None
    """
    自定义类名
    """

    default_value: Any = [None, None]
    """
    默认选中项
    """

    disabled: Any = None
    """
    是否禁用
    """

    format: str = "YYYY-MM-DD"
    """
    日期格式
    """

    popup_class_name: str = None
    """
    弹出日历的额外类名
    """

    input_read_only: bool = False
    """
    输入框是否只读
    """

    locale: Any = None
    """
    国际化配置
    """

    mode: str = None
    """
    日期面板的状态
    """

    next_icon: Any = None
    """
    自定义下一个图标
    """

    open: bool = None
    """
    控制浮层显隐
    """

    picker: str = "date"
    """
    选择器类型
    """

    placeholder: List[str] = ["开始日期", "结束日期"]
    """
    输入框占位文本
    """

    placement: str = None
    """
    浮层预设位置
    """

    popup_style: Any = None
    """
    弹出日历的额外样式
    """

    prev_icon: Any = None
    """
    自定义上一个图标
    """

    size: str = None
    """
    输入框大小
    """

    status: str = None
    """
    校验状态
    """

    style: Dict[str, Any] = None
    """
    自定义样式
    """

    suffix_icon: Any = None
    """
    自定义选择框后缀图标
    """

    super_next_icon: Any = None
    """
    自定义 << 切换图标
    """

    super_prev_icon: Any = None
    """
    自定义 >> 切换图标
    """

    value: Any = None
    """
    指定选中项
    """

    default_picker_value: str = None
    """
    默认面板日期
    """

    show_now: bool = None
    """
    设定 showTime 时，是否显示“此刻”按钮
    """

    show_time: Any = None
    """
    是否增加时间选择功能
    """

    show_today: bool = None
    """
    是否展示“今天”按钮
    """

    def set_allow_clear(self, allow_clear: bool):
        self.allow_clear = allow_clear
        return self  # 设置是否支持清除并返回自身

    def set_auto_focus(self, auto_focus: bool):
        self.auto_focus = auto_focus
        return self  # 设置是否自动聚焦并返回自身

    def set_bordered(self, bordered: bool):
        self.bordered = bordered
        return self  # 设置是否显示边框并返回自身

    def set_class_name(self, class_name: str):
        self.class_name = class_name
        return self  # 设置自定义类名并返回自身

    def set_default_value(self, default_value: Any):
        self.default_value = default_value
        return self

    def set_value(self, value: Any):
        self.value = value
        return self

    def set_format(self, format: str):
        self.format = format
        return self  # 设置日期格式并返回自身

    def set_popup_class_name(self, popup_class_name: str):
        self.popup_class_name = popup_class_name
        return self  # 设置弹出日历类名并返回自身

    def set_input_read_only(self, input_read_only: bool):
        self.input_read_only = input_read_only
        return self  # 设置输入框只读并返回自身

    def set_locale(self, locale: Any):
        self.locale = locale
        return self  # 设置国际化配置并返回自身

    def set_mode(self, mode: str):
        self.mode = mode
        return self  # 设置日期面板模式并返回自身

    def set_next_icon(self, next_icon: Any):
        self.next_icon = next_icon
        return self  # 设置下一个图标并返回自身

    def set_open(self, open: bool):
        self.open = open
        return self  # 设置浮层显隐并返回自身

    def set_picker(self, picker: str):
        self.picker = picker
        return self  # 设置选择器类型并返回自身

    def set_placeholder(self, placeholder: List[str]):
        self.placeholder = placeholder
        return self  # 设置输入框占位文本并返回自身

    def set_placement(self, placement: str):
        self.placement = placement
        return self  # 设置浮层位置并返回自身

    def set_popup_style(self, popup_style: Any):
        self.popup_style = popup_style
        return self  # 设置弹出日历样式并返回自身

    def set_prev_icon(self, prev_icon: Any):
        self.prev_icon = prev_icon
        return self  # 设置上一个图标并返回自身

    def set_size(self, size: str):
        self.size = size
        return self  # 设置输入框大小并返回自身

    def set_status(self, status: str):
        self.status = status
        return self  # 设置校验状态并返回自身

    def set_suffix_icon(self, suffix_icon: Any):
        self.suffix_icon = suffix_icon
        return self  # 设置后缀图标并返回自身

    def set_super_next_icon(self, super_next_icon: Any):
        self.super_next_icon = super_next_icon
        return self  # 设置 << 切换图标并返回自身

    def set_super_prev_icon(self, super_prev_icon: Any):
        self.super_prev_icon = super_prev_icon
        return self  # 设置 >> 切换图标并返回自身

    def set_default_picker_value(self, default_picker_value: str):
        self.default_picker_value = default_picker_value
        return self  # 设置默认面板日期并返回自身

    def set_show_now(self, show_now: bool):
        self.show_now = show_now
        return self  # 设置是否显示“此刻”按钮并返回自身

    def set_show_time(self, show_time: Any):
        self.show_time = show_time
        return self  # 设置是否增加时间选择功能并返回自身

    def set_show_today(self, show_today: bool):
        self.show_today = show_today
        return self  # 设置是否展示“今天”按钮并返回自身



import json
from typing import Any, Dict, List

from tortoise import Model

from quark import Message, Request, Resource, models, utils
from quark.app import actions
from quark.component.form import field
from quark.component.form.rule import Rule
from quark.services.auth import AuthService
from quark.services.user import UserService


class Account(Resource):
    """
    个人设置
    """

    async def init(self, request: Request):

        # 页面标题
        self.title = "个人设置"

        # 模型
        self.model = models.User

        return self

    # 字段
    async def fields(self, request: Request) -> List[Any]:

        return [
            field.image("avatar", "头像"),
            field.text("nickname", "昵称").set_rules([Rule.required("昵称必须填写")]),
            field.text("email", "邮箱").set_rules([Rule.required("邮箱必须填写")]),
            field.text("phone", "手机号").set_rules([Rule.required("手机号必须填写")]),
            field.radio("sex", "性别")
            .set_options(
                [
                    field.radio_option("男", 1),
                    field.radio_option("女", 2),
                ]
            )
            .set_default_value(1),
            field.password("password", "密码")
            .set_rules(
                [
                    Rule.regexp(r"/^.{6,}$/", "密码不少于六位"),
                    Rule.regexp(r"/.*[A-Z].*/", "至少包含一个大写字母"),
                    Rule.regexp(r"/.*[a-z].*/", "至少包含一个小写字母"),
                    Rule.regexp(r"/.*[0-9].*/", "至少包含一个数字"),
                    Rule.regexp(
                        r"/.*[!@#$%^&*()_+{}\[\]:;<>,.?~\\/-].*/",
                        "至少包含一个特殊字符",
                    ),
                ]
            )
            .set_help(
                "密码不少于六位，且至少包含一个大写字母、小写字母、数字和特殊字符"
            ),
        ]

    # 行为
    async def actions(self, request: Request) -> List[Any]:
        return [
            actions.FormSubmit(),
            actions.FormReset(),
            actions.FormBack(),
            actions.FormExtraBack(),
        ]

    # 表单显示前回调
    async def before_form_showing(self, request: Request) -> Any:
        admin_info = await AuthService(request).get_current_admin()
        user = await UserService().get_info_by_id(admin_info.id)
        user_dict = user.__dict__.copy()
        user_dict.pop("password", None)  # 安全地移除密码字段
        return user_dict

    # 表单提交处理
    async def form_handle(
        self, request: Request, model: Model, data: Dict[str, Any]
    ) -> Any:
        # 头像处理
        if data.get("avatar"):
            data["avatar"] = json.dumps(data["avatar"], ensure_ascii=False)

        # 加密密码
        if data.get("password"):
            data["password"] = utils.hash_password(data["password"])

        try:
            # 获取登录管理员信息
            admin_info = await AuthService(request).get_current_admin()
            model = await UserService().get_info_by_id(admin_info.id)
            await model.update_from_dict(data)
        except Exception as e:
            return Message.error(str(e))

        return Message.success("操作成功")



from typing import Any, Dict, List

from quark import Request, Resource, models
from quark.app import actions, searches
from quark.component.form import field
from quark.component.form.rule import Rule
from quark.services import MenuService, PermissionService


class Menu(Resource):
    """
    菜单管理
    """

    async def init(self, request: Request):
        self.title = "菜单"
        self.model = models.Menu
        self.table_list_to_tree = True
        self.query_order = ["sort"]
        self.page_size = False

        return self

    # 字段
    async def fields(self, request: Request) -> List[Any]:

        # 权限列表
        permissions = await PermissionService().data_source()

        # 菜单列表
        menus = await MenuService().get_list_with_root()

        return [
            field.hidden("id", "ID"),  # 列表读取且不展示的字段
            field.hidden("pid", "PID").only_on_index(),  # 列表读取且不展示的字段
            field.hidden("query", "查询参数"),
            field.hidden("api", "API接口"),
            field.hidden("url", "URL地址"),
            field.hidden("component", "组件"),
            field.group(
                [
                    field.text("name", "名称").set_rules(
                        [Rule.required("名称必须填写")]
                    ),
                    field.text("guard_name", "守卫")
                    .set_rules([Rule.required("守卫必须填写")])
                    .set_default_value("admin")
                    .only_on_forms(),
                    field.radio("type", "类型")
                    .set_options(
                        [
                            field.radio_option("目录", 1),
                            field.radio_option("菜单", 2),
                            field.radio_option("按钮", 3),
                        ]
                    )
                    .set_rules([Rule.required("类型必须选择")])
                    .set_default_value(1),
                ]
            ),
            field.group(
                [
                    field.icon("icon", "图标").only_on_forms(),
                    field.number("sort", "排序")
                    .set_editable(True)
                    .set_default_value(0),
                    field.tree_select("pid", "上级菜单")
                    .set_tree_data(menus, -1, "pid", "name", "id")
                    .set_default_value(0)
                    .only_on_forms(),
                ]
            ),
            field.dependency().set_when(
                "type",
                1,
                lambda: [
                    field.text("path", "路由")
                    .set_rules([Rule.required("路由必须填写")])
                    .set_editable(True)
                    .set_help("访问的路由地址，如：`user`")
                    .set_width("400px"),
                ],
            ),
            field.dependency().set_when(
                "type",
                2,
                lambda: [
                    field.radio("page_type", "页面类型")
                    .set_options(
                        [
                            field.radio_option("默认", 1),
                            field.radio_option("引擎", 2),
                            field.radio_option("外链", 3),
                            field.radio_option("iframe", 4),
                        ]
                    )
                    .set_rules([Rule.required("页面类型必须选择")])
                    .set_default_value(1)
                    .only_on_forms(),
                    field.dependency().set_when(
                        "page_type",
                        1,
                        lambda: [
                            field.text("path", "路由地址")
                            .set_rules([Rule.required("路由地址必须填写")])
                            .set_editable(True)
                            .set_help("访问的路由地址，如：`user`")
                            .set_width("400px")
                            .only_on_forms(),
                            field.text("component", "组件路径")
                            .set_rules([Rule.required("组件路径必须填写")])
                            .set_editable(True)
                            .set_help("访问的组件路径，如：`user/index`")
                            .set_width("400px")
                            .only_on_forms(),
                        ],
                    ),
                    field.dependency().set_when(
                        "page_type",
                        2,
                        lambda: [
                            field.text("path", "路由地址")
                            .set_rules([Rule.required("路由地址必须填写")])
                            .set_editable(True)
                            .set_help("访问的路由地址，如：`user`")
                            .set_width("400px")
                            .only_on_forms(),
                            field.text("api", "接口地址")
                            .set_rules([Rule.required("接口地址必须填写")])
                            .set_editable(True)
                            .set_help("引擎接口地址，如：`/api/admin/user/index`")
                            .set_width("400px")
                            .only_on_forms(),
                        ],
                    ),
                    field.dependency().set_when(
                        "page_type",
                        3,
                        lambda: [
                            field.text("path", "外链地址")
                            .set_rules([Rule.required("外链地址必须填写")])
                            .set_editable(True)
                            .set_help("访问的外链地址，以`http(s)://`开头")
                            .set_width("400px")
                            .only_on_forms(),
                        ],
                    ),
                    field.dependency().set_when(
                        "page_type",
                        4,
                        lambda: [
                            field.text("path", "路由地址")
                            .set_rules([Rule.required("路由地址必须填写")])
                            .set_editable(True)
                            .set_help("访问的路由地址，如：`user`")
                            .set_width("400px")
                            .only_on_forms(),
                            field.text("url", "iframe地址")
                            .set_rules([Rule.required("iframe地址必须填写")])
                            .set_editable(True)
                            .set_help("访问的iframe地址，以`http(s)://`开头")
                            .set_width("400px")
                            .only_on_forms(),
                        ],
                    ),
                ],
            ),
            field.dependency().set_when(
                "type",
                ">",
                1,
                lambda: [
                    field.text("permission", "权限标识")
                    .set_help("鉴权标识，如：`user:index`")
                    .set_width("400px"),
                ],
            ),
            field.group(
                [
                    field.switch("visible", "显示")
                    .set_true_value("显示")
                    .set_false_value("隐藏")
                    .set_editable(True)
                    .set_default_value(True),
                    field.switch("status", "状态")
                    .set_true_value("正常")
                    .set_false_value("禁用")
                    .set_editable(True)
                    .set_default_value(True),
                ]
            ),
            field.dependency().set_when(
                "type",
                3,
                lambda: [
                    field.transfer("permission_ids", "绑定权限")
                    .set_data_source(permissions)
                    .set_list_style(
                        {
                            "width": "320px",
                            "height": "300px",
                        }
                    )
                    .set_show_search(True)
                    .only_on_forms(),
                ],
            ),
        ]

    # 搜索
    async def searches(self, request: Request) -> List[Any]:
        return [
            searches.Input("name", "名称"),
            searches.Input("path", "路由"),
            searches.Status(),
        ]

    # 行为
    async def actions(self, request: Request) -> List[Any]:
        return [
            actions.MenuCreateDrawer(self),
            actions.BatchDelete(),
            actions.ChangeStatus(),
            actions.MenuEditDrawer(self),
            actions.Delete(),
        ]

    # 编辑页面显示前回调
    async def before_editing(
        self, request: Request, data: Dict[str, Any]
    ) -> Dict[str, Any]:
        id = request.query_params.get("id", "")
        if id and isinstance(id, str):
            try:
                id_int = int(id)
                permission_ids = []
                permissions = await PermissionService().get_menu_permissions(id_int)
                for v in permissions:
                    permission_ids.append(v.id)
                data["permission_ids"] = permission_ids
            except ValueError:
                pass

        page_type = data.get("page_type", 0)
        if page_type == 2:
            import json

            query = json.loads(data.get("query", "{}"))
            data["api"] = query.get("api")
        elif page_type == 4:
            import json

            query = json.loads(data.get("query", "{}"))
            data["url"] = query.get("url")

        return data

    async def before_saving(self, request, submit_data):
        if submit_data.get("page_type", 0) == 2:
            submit_data["query"] = f'{{"api":"{submit_data.get("api", "")}"}}'
        if submit_data.get("page_type", 0) == 4:
            submit_data["query"] = f'{{"url":"{submit_data.get("url", "")}"}}'

        return submit_data

    # 保存后回调
    async def after_saved(self, request, id, data, result):
        if data.get("permission_ids") is not None:
            await PermissionService().add_menu_permission(id, data["permission_ids"])
